             !  Computes the transition between initial and final steady states
   
!**********************************************************************************************************************************
!PARAMETERS
!**********************************************************************************************************************************
 
      module parameters_mod
      implicit none
   
      integer, parameter :: ncapt = 20  !Guess at length of transition 
      integer, parameter :: nhpts = 150 
      integer, parameter :: hadd = 9
      integer, parameter :: nh2pts = 10000
      integer, parameter :: nkpts = 2
      integer, parameter :: napts = 9
      integer, parameter :: nlpts = 300
      integer, parameter :: nread4 = 1 ! reads in transition guess
      integer, parameter :: mxiter = 300
      integer, parameter :: mxiter2 = 500
      integer, parameter :: mxpol = 200
      integer, parameter :: nlzpts = 500
      integer, parameter :: iwsize = 50
      integer, parameter :: nwsize = 500
      integer, parameter :: nparam = 2
      integer, parameter :: neq = 0
      integer, parameter :: nf = 1
      integer, parameter :: nineq = 1
      integer, parameter :: nineqn = 1
      integer, parameter :: neqn = 0
      integer, parameter :: ncal = 4
      integer, parameter :: bigaswitch = 0  ! 0 - Wilson Case, 1 - MLK Case
      real*8, parameter :: costm = 0.0D+00  ! Set to a very large number for X case
      
      real*8, parameter :: one = 1.0D+00
      real*8, parameter :: biga2 = 5.00D+00 
      real*8, parameter :: mu = 0.64D+00
      real*8, parameter :: beta = 0.665D+00
      real*8, parameter :: wfix = 1.0D+00
      real*8, parameter :: nuc = 1.5D+00
      real*8, parameter :: nus = 2.0D+00 
      real*8, parameter :: pmin =  1.0D-06
      real*8, parameter :: hhom = 0.00D+00
      real*8, parameter :: phom = 0.0001D+00 
      real*8, parameter :: stig1 = 1.0D-03
      real*8, parameter :: stig2 = 1.0D-03
      real*8, parameter :: bighmin = 1.0D-02
      real*8, parameter :: lfix1 = 0.114D+00
      real*8, parameter :: lfix2 = one-lfix1
      real*8, parameter :: htol  = 3.0D-01
      real*8, parameter :: ptol = 1.0D-01 
      real*8, parameter :: lam = one/3.0D+00
      
      
      real*8, dimension(napts,napts) :: pi
      real*8, dimension(nlpts) :: lpts

      end module parameters_mod
      
      module states_mod
        use parameters_mod
        implicit none
        integer :: nbdstay,nbdgo,nap,ntp,nt,nk,nh
        real*8 :: xhcur,pstay,pgo,pop1,pop2,trans,wage,bighstay,bighgo
        real*8 :: siga,rho,theta,gam1,biga1
        real*8 :: cont,phit,invt,ht,vtrans
        real*8, dimension(nhpts,napts,nkpts,ncapt+1) :: y2g,opthg,optkg,optcg,optphig,optig
        real*8, dimension(nhpts) :: hpts
        real*8, dimension(nhpts,napts,nkpts) :: vhdp
        real*8, dimension(nhpts,ncapt) :: upbnd1,upbnd2
        real*8, dimension(nh2pts) :: h2pts
        real*8, dimension(napts) :: apts
        real*8, dimension(nkpts) :: kpts
        real*8, dimension(nh2pts,napts,nkpts) :: f1,f1hat,vgrid
        real*8, dimension(nhpts,napts,nkpts) :: v,vdp,welf,ceq,phieq,ieq,ystar,hpeq
        real*8, dimension(nkpts,ncapt) :: hcap,hhat,pcap,phat
        real*8, dimension(nkpts,ncapt) :: bighum,bigc,bigq,meana,pops,hp,bigi
        real*8, dimension(nhpts,napts,nkpts,nlpts) :: vhat,vhat2
        
      end module states_mod
      
      
!**********************************************************************************************************************************
!MAIN PROGRAM
!**********************************************************************************************************************************    
        program main
        use parameters_mod
        use states_mod
        implicit none  
        integer :: i,j,jj,k,kk,kkk,kkz,multpl,dumi,khi,klo
        real*8 :: hinc,zeta2,pcap1inc,pcap2inc,hcap1inc,hcap2inc,littlehouse,zbrent,dum,eusum
        real*8, dimension(nh2pts,napts,nkpts) :: fstart
        real*8, dimension(ncal) :: xcal
        real*8 :: yval,yp,ydp,wcalc,getxhp
        real*8, dimension(nkpts) :: bighstar
     interface
         double precision function welfare(x)
            use parameters_mod
            use states_mod
            implicit none
            real*8, intent(in) :: x
         end function welfare
      end interface

   
!***********************************************************     
!Build human capital grids
!***********************************************************
   
      open(2,file='hgrid.in',form='formatted')
        do i = 1,nhpts
            read(2,458) hpts(i)
458         format(f18.10)
        end do
      close(2)
      
      open(3,file='nbd_calib.best',form='formatted')
      do i = 1,ncal
         read(3,132) xcal(i),dum
 132     format(2f18.10)
      end do
      close(3)
      
      biga1  = xcal(1)
      siga   = xcal(2)
      theta = xcal(3)
      gam1  = xcal(4)
      
      h2pts(1) = hpts(1)
      hinc = (hpts(nhpts)-hpts(1))/dble(real(nh2pts-1))
      do i = 2,nh2pts
         h2pts(i) = h2pts(i-1)+hinc
      end do
      
      kpts(1) = one
      kpts(2) = 2.0D+00
    
      rho = 0.0D+00
      
      apts(1) = -sqrt(dble(napts-1))*siga/sqrt(one-rho*rho)
      apts(napts) = -apts(1)
      do i = 2,napts-1
         apts(i) = apts(i-1)+(apts(napts)-apts(1))/dble(napts-1)
      end do
      apts = dexp(apts)
      
      call rouwenhorst(rho,pi,napts)

!***********************************************************     
! Value function and decision rules
!***********************************************************


     open(3,file='saving_fin.out',form='formatted')
        do k = 1,nkpts
          do j = 1,napts
            do i = 1,nhpts
            read(3,456) dum,dum,dum,y2g(i,j,k,ncapt+1),dum,opthg(i,j,k,ncapt+1),optig(i,j,k,ncapt+1),optcg(i,j,k,ncapt+1),optphig(i,j,k,ncapt+1),optkg(i,j,k,ncapt+1),dum,dum
456         format(2f12.6,f5.1,6f18.10,f5.1,f18.10,i4)
            end do

         end do
      end do
      close(3)
      
!***********************************************************     
! Distribution of human capital
!***********************************************************
         open(2,file='saving.fst',form='formatted')
         do k = 1,nkpts
            do i = 1,nh2pts
               read(2,547) dum,dum,f1(i,:,k)
 547           format(11f18.12)            
            end do
         end do
         close(2)
         
         fstart = f1

!***********************************************************     
! Joint human capital stocks
!***********************************************************
     open(2,file='saving_init.agg',form='formatted')
        do k = 1,nkpts
          read(2,391) dum,dum,hcap(k,1),dum,pcap(k,1),dum,dum,dum,dum 
391       format(i4,9f16.10)
        end do
     close(2)
     
     bighstar(1) = hcap(1,1)
     bighstar(2) = hcap(2,1)
     
     open(2,file='saving_fin.agg',form='formatted')
        do k = 1,nkpts
          read(2,391) dum,dum,hcap(k,ncapt),dum,pcap(k,ncapt),dum,dum,dum,dum
        end do
     close(2)
    
    
      open(4,file='saving_init.out',form='formatted')
      do k = 1,nkpts
         do j = 1,napts
            do i = 1,nhpts
               read(4,208) dum,dum,dum,dum,hpeq(i,j,k),ieq(i,j,k),ceq(i,j,k),phieq(i,j,k),dum,dum,dum,dum
 208           format(2f12.6,f5.1,5f18.10,f5.1,2f18.10,i4) 
            end do
         end do
      end do
      close(4)
       
     if (nread4 .eq. 1) then
 !       open(1,file='saving.trn2',form='formatted')
 !               do i = 1,ncapt
	!              read(1,951) dumi, pcap(1,i), pcap(2,i), hcap(1,i), hcap(2,i),dum, dum,dum,dum,dum,dum,dum,dum,dum,dum
 !!951               format(i5,14f18.10)
 !               end do
 !               close(1) 
          
          open(2,file='saving.trn',form='formatted')
        do i = 1,ncapt
          read(2,938)  dum, pcap(1,i), pcap(2,i), hcap(1,i), hcap(2,i)
        end do
        close(2)
        
     else   
        pcap1inc = (pcap(1,ncapt)-pcap(1,1))/(ncapt-1) 
        pcap2inc = (pcap(2,ncapt)-pcap(2,1))/(ncapt-1)
     
        hcap1inc = (hcap(1,ncapt)-hcap(1,1))/(ncapt-1)
        hcap2inc = (hcap(2,ncapt)-hcap(2,1))/(ncapt-1)
   
        do i = 2,ncapt-1
            pcap(1,i) = pcap(1,i-1)+pcap1inc
            pcap(2,i) = pcap(2,i-1)+pcap2inc
            hcap(1,i) = hcap(1,i-1)+hcap1inc
            hcap(2,i) = hcap(2,i-1)+hcap2inc
        end do
      
    endif 
       
         
      do i = 1,ncapt
        write(6,937) i,pcap(:,i), hcap(:,i)
      end do
      pause
 
 937  format(i5,4f18.12)
 
      do kkk = 1,1000
         do kkz = 1,20000
         call transpath(pcap,hcap,phat,hhat,fstart)
          	if (multpl(kkk,1) .eq. 1) then
	        write(6,322) kkk,maxval(dabs(pcap(1,:)-phat(1,:))),maxval(dabs(pcap(2,:)-phat(2,:))),maxval(dabs(hcap(1,:)-hhat(1,:))),maxval(dabs(hcap(2,:)-hhat(2,:))),phat(1,ncapt)-pcap(1,ncapt),phat(2,ncapt)-pcap(2,ncapt)
 322        format(i5,6f12.8)
         endif
                       
	     if (maxval(dabs(pcap-phat)) .lt. ptol)  goto 8 
         print*, 'updating prices', kkz, kkk  
             if (maxval(dabs(pcap-phat)) .lt. 0.2D+00)  then
                open(1,file='saving.trn2',form='formatted')
                do i = 1,ncapt
	              write(1,951) i, pcap(1,i), pcap(2,i), hcap(1,i), hcap(2,i), hp(1,i), hp(2,i),bigi(1,i),bigi(2,i),bigq(1,i),bigq(2,i),bigc(1,i),bigc(2,i),pops(1,k),pops(2,k)
 951               format(i5,14f18.10)
                end do
                close(1)  
             endif
             pcap = (one-phom)*pcap+phom*phat 
          end do
  8      hcap = (one-hhom)*hcap+hhom*hhat
         print*, 'updating H', kkk, 1000   
         open(1,file='saving.trn',form='formatted')
         do i = 1,ncapt
	        write(1,938) i,pcap(1,i),pcap(2,i),hcap(1,i),hcap(2,i)
         end do
         close(1)   

         if (maxval(dabs(hcap-hhat)) .lt. htol) goto 1
      end do

 1    open(1,file='saving.trn',form='formatted')
      do i = 1,ncapt
	     write(1,938) i,pcap(1,i),pcap(2,i),hcap(1,i),hcap(2,i)
 938     format(i5,4f18.10)
      end do
      close(1)
      
      
           if ((maxval(dabs(hcap-hhat)) .lt. htol) .and. (maxval(dabs(pcap-phat)) .lt. ptol)) goto 2
            print*, '----------------------------------------------'
            print*, '----------------------------------------------'
 328        format(f15.8)
 329        format(2f15.8)
 330        format(3f15.8)
  
         
2     print*, 'going to welfare'
      
      lpts(1) = -0.35D+00
      lpts(nlpts) = 0.50D+00
          
      do i = 2,nlpts-1
         lpts(i) = lpts(i-1) + (lpts(nlpts)-lpts(1))/dble(real(nlpts-1))  
      end do
        
      lpts(nlpts) = 5.0D+00
      
      vhat2 = 1.0D+00
      
      do kkk = 1,mxiter
        vhat = vhat2
           do kk = 1,nlpts
             call interp(hpts,vhat(:,:,:,kk),vhdp)
             do k = 1,nkpts
                  do j = 1,napts
                    do i = 1,nhpts
                        cont = ceq(i,j,k)
                        phit = phieq(i,j,k)
                        ht = hpeq(i,j,k)                
                        eusum = 0.0D+00
                        do jj = 1,napts
                           call splint2(hpts,vhat(:,:,:,kk),vhdp,ht,yval,yp,ydp,khi,klo,0,nhpts,1,0,0,jj,k)
                           eusum = eusum + pi(j,jj)*yval
                        end do                     
                        vhat2(i,j,k,kk) = (((one+lpts(kk))*cont)**(one-nuc))/(one-nuc)+theta*(phit**(one-nus))/(one-nus)+ beta*eusum
                    end do
                  end do
              end do
          end do
            if (maxval(dabs(vhat2-vhat)) < 1.0D-08) then
              goto 12
            endif
            if (multpl(kkk,1) .eq. 1) then
              print*, kkk,maxval(dabs(vhat2-vhat))
            endif
      end do     
      
 12   open(1,file='nbd_trans.wel',form='formatted')
      do k = 1,nkpts
         nk = k
         do i = 1,nhpts
           nh = i
            do j = 1,napts
               nap = j
               vtrans = ystar(i,j,k)
               wcalc = zbrent(welfare,lpts(1),lpts(nlpts),1.0D-08)
               welf(i,j,k) = wcalc
            end do 
            write(1,160) kpts(k),hpts(i),welf(i,1:napts,k)
160         format(f4.2,10f18.10)      ! Change if napts changes
         end do
      end do
      close(1)

      open(3,file='vtrans.out',form='formatted')
      do k = 1,nkpts
         do i = 1,nhpts
            write(3,981) k,hpts(i),ystar(i,1:napts,k) ! Change if napts changes
 981        format(i4,10f18.10)
         end do
      end do
      close(3)
      
      open(3,file='trans_path.out',form = 'formatted')
      do i = 1,ncapt
         write(3,939) i, pcap(1,i), pcap(2,i), hcap(1,i), hcap(2,i), hp(1,i), hp(2,i),bigq(1,i),bigq(2,i),bigc(1,i),bigc(2,i),pops(1,i),pops(2,i)
 939     format(i4,12f18.10)
      end do
      close(3)
      
      do i = 1,ncapt
         write(6,938) i, pcap(1,i), pcap(2,i), hcap(1,i), hcap(2,i) 
      end do
      
            
      print*, 'all done'
      
      pause
 
      end
  
  
  
!**********************************************************************************************************************************
!Subroutines and Functions
!**********************************************************************************************************************************  
      subroutine transpath(pval,hcapval,pnew,hcapnew,fstart)
      use parameters_mod
      use states_mod
	  implicit none
	  real*8, dimension(nh2pts,napts,nkpts) :: fstart,fnrm
	  real*8, dimension(nh2pts,napts,nkpts) :: opth2,optc2,optphi2,opti2,optk2
	  real*8, dimension(nh2pts,napts,nkpts,ncapt+1) :: ft,fthat
	  real*8, dimension(nhpts,napts,nkpts,ncapt) :: val1,val2
	  real*8, dimension(nhpts,napts,nkpts) :: vcur
	  real*8, dimension(nkpts,ncapt) :: pnew,hcapnew,pval,hcapval
	  integer :: i,j,k,kkk,iii,jlo1,jlo2,multpl,mode,iprint,miter,inform,jk,kk,ierr
	  real*8 :: wgt1,wgt2,fsum,eps,epsneq,udelta,bigbnd,ck,gk,kstay,kgo,neps &
	            ,cstay,cgo,hstay,hgo,istay,igo,ystay,ygo,w,phistay,phigo,bigh1,bigh2,bigq1,bigq2 &
	            ,bigc1,bigc2,bigqcheck,littlehouse,getxhp,upcstay,upcgo,zbrent2,stig,hp1,hp2,bigi1,bigi2,test
      real*8, dimension(nwsize) :: work
      real*8, dimension(nparam) :: x,bl,bu,bl2,bu2,bcheck
      real*8, dimension(nf+1) :: ff
      real*8, dimension(nineq+neq+1) :: g
      integer, dimension(nwsize) :: iw
      external func,func2,const,const2,dconst,dconst2,grobfd,grcnfd,dfunc,dfunc2
      interface
      double precision function upperb(x)
        real*8, intent(in) :: x
      end function upperb
      end interface
      
      mode = 100
      iprint = 0
      miter = 500
      eps = 1.0D-12
      epsneq = 1.0D-12
      udelta = 0.0D+00
	   bigbnd= 1.0D+10
     
      wage = wfix
      littlehouse = 1.0D-08
      
      do kkk = 1,ncapt
         if (nuc .eq. 1.0D+00) then
            do i = 1,nhpts
               xhcur = hpts(i)
               upbnd1(i,kkk) = 0.9999D+00*wfix*xhcur
               upbnd2(i,kkk) = upbnd1(i,kkk)
            end do
         else  
            do i = 1,nhpts
               xhcur = hpts(i)
               upbnd1(i,kkk) = zbrent2(upperb,1.0D-08,wfix*xhcur,pcap(1,kkk),1.0D-08)
               upbnd2(i,kkk) = zbrent2(upperb,1.0D-08,wfix*xhcur,pcap(2,kkk),1.0D-08)
            end do
         endif
      end do

      do kkk = ncapt,1,-1
      
      open(5,file='upperbounds.out',form='formatted')
      do i = 1,nhpts
      write(5,204) upbnd1(i,kkk),upbnd2(i,kkk), pcap(:,kkk)
 204  format(4f18.10)
      end do
      close(5)
      
        v = 0.0D+00
        nt = kkk
        ntp = nt+1
        do k = 1,nkpts
           do i = 1,napts
              do j = 1,napts
                 v(:,i,k) = v(:,i,k)+pi(i,j)*y2g(:,j,k,ntp)
              end do
           end do
        end do 
        
        call interp(hpts,v,vdp)
  
        do j = 1,napts
            nap = j
            do k = 1,nkpts
               nk = k
                if (k .eq. 1) then
                    kstay = 1.0D+00         
                    kgo = 2.0D+00            
                    bighstay = hcap(1,kkk)     
                    bighgo = hcap(2,kkk)
                    pstay = pcap(1,kkk)
                    pgo = pcap(2,kkk)
                    nbdstay = 1
                    nbdgo = 2
                elseif (k .eq. 2) then
                    kstay = 2.0D+00
                    kgo = 1.0D+00
                    bighstay = hcap(2,kkk)     
                    bighgo = hcap(1,kkk)
                    pstay = pcap(2,kkk)
                    pgo = pcap(1,kkk)
                    nbdstay = 2
                    nbdgo = 1
                endif              
                 
	        do i = 1,nhpts
               
               if (k .eq. 1) then 
                   upcstay = upbnd1(i,kkk)
                   upcgo = upbnd2(i,kkk)
               else
                   upcstay = upbnd2(i,kkk)
                   upcgo = upbnd1(i,kkk)
               endif  
                 
 	           xhcur = hpts(i)
               bl(1) = 1.0D-08
               bl(2) = 1.0D-08
                
               bu(1) = wage*xhcur*0.9999D+00-pstay*littlehouse
               bu(2) = upcstay
 
               bcheck = bu-bl

                if (minval(bcheck) .lt. 1.0D-08) then
                   print*, 'error on constraint set for staying'
                   print*, bl
                   print*, bu
                   print*, wage,xhcur,pstay
                   print*, upbnd1(i,kkk),upbnd2(i,kkk)
                   pause
                endif

                  x(1) = optig(i,j,k,kkk+1)
                  x(2) = optcg(i,j,k,kkk+1)

               call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, & 
                          epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,func,const,dfunc,dconst)
   
               istay = x(1)
               cstay = x(2)
               hstay = getxhp(istay,bighstay,apts(nap),nbdstay)
               
               phistay = (theta/pstay)**(one/nus)*(x(2)**(nuc/nus))
               ystay = -ff(1)
               
               
                  x(1) = optig(i,j,k,kkk+1)
                  x(2) = optcg(i,j,k,kkk+1)
               
                bl2(1) = 1.0D-08
                bl2(2) = 1.0D-08
                
                bu2(1) = wage*xhcur*0.9999D+00-pgo*littlehouse-costm
                bu2(2) = upcgo-costm
                
                bcheck = bu2-bl2
                
             if (minval(bcheck) .lt. 1.0D-08) then
                   cgo = 1.0D-08
                   igo = 1.0D-08
                   phigo = (theta/pgo)**(one/nus)*(cgo**(nuc/nus))
                   ygo = -1.0D+06
                   hgo = hpts(1)
              else
               call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
                          epsneq,udelta,bl2,bu2,x,ff,g,iw,iwsize,work,nwsize,func2,const2,dfunc2,dconst2)
!                          
!               call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
!                          epsneq,udelta,bl2,bu2,x,ff,g,iw,iwsize,work,nwsize,func2,const2,grobfd,dconst2)
               
               igo = x(1)
               cgo = x(2)
               hgo = getxhp(igo,bighgo,apts(nap),nbdgo)
               phigo = (theta/pgo)**(one/nus)*(x(2)**(nuc/nus))
               ygo = -ff(1)          
            endif        


             if (kstay .lt. 1.5D+00) then
                stig = stig1
                val1(i,j,k,kkk) = ystay
                val2(i,j,k,kkk) = ygo
                if (val2(i,j,k,kkk) .lt. -1.0D+05) then 
                   val2(i,j,k,kkk) = 0.0D+00
                endif        
             else 
                stig = stig2
                val1(i,j,k,kkk) = ygo
                val2(i,j,k,kkk) = ystay
                if (val1(i,j,k,kkk) .lt. -1.0D+05) then 
                   val1(i,j,k,kkk) = 0.0D+00
                endif  
             endif
             

               if (ystay .gt. ygo - stig) then 
                  optig(i,j,k,kkk) = istay
                  opthg(i,j,k,kkk) = hstay
                  optcg(i,j,k,kkk) = cstay
                  optkg(i,j,k,kkk) = kstay
                  phistay = (theta/pstay)**(one/nus)*(cstay**(nuc/nus))
                  optphig(i,j,k,kkk) = phistay
                  y2g(i,j,k,kkk) = ystay 
                  vcur(i,j,k) = ystay  
               endif
                 if (ygo .gt. ystay + stig) then
                  optig(i,j,k,kkk) = igo
                  opthg(i,j,k,kkk) = hgo
                  optcg(i,j,k,kkk) = cgo
                  optkg(i,j,k,kkk) = kgo
                  phigo = (theta/pgo)**(one/nus)*(cgo**(nuc/nus))
                  optphig(i,j,k,kkk) = phigo
                  y2g(i,j,k,kkk) = ygo
                  vcur(i,j,k) = ygo
                endif
    
                end do
            end do
         end do
         
         if (multpl(kkk,10) .eq. 1) then
	      write(6,773) kkk
 773        format('transition period ',i5)
         endif

      end do
     
        open(2,file='vopt.out',form='formatted')
        do kk = 1,ncapt
          do k = 1,nkpts
             do j = 1,napts
                do i = 1,nhpts
                    write(2,902) kk, hpts(i), apts(j), kpts(k),optkg(i,j,k,kk),y2g(i,j,k,kk),val1(i,j,k,kk),val2(i,j,k,kk),opthg(i,j,k,kk), optig(i,j,k,kk), optcg(i,j,k,kk), optphig(i,j,k,kk) 
902                 format(i4,2f18.10,2f5.1,7f18.10)
                end do
             end do
          end do
        end do
        close(2)
           
        open(2,file='vdiff.out',form='formatted')
        do kk = 1,ncapt
          do k = 1,nkpts
             do j = 1,napts
                do i = 1,nhpts
                    write(2,906) kk, hpts(i), apts(j), kpts(k), optkg(i,j,k,kk),val2(i,j,k,kk)-val1(i,j,k,kk)
906                 format(i4,2f18.10,2f5.1,1f18.10)
                end do
             end do
          end do
        end do
        close(2) 
      
      ystar = vcur
      
      ft = 0.0D+00
      ft(:,:,:,1) = fstart
	  fthat = 0.0D+00
	
      do kkk = 1,ncapt
   
	     ! GET THE INTRATEMPORAL DISTRIBUTION 
	     do i = 1,nh2pts
	        do j = 1,napts
	           do k = 1,nkpts
	             call hunt(hpts,nhpts,h2pts(i),jlo1)
	             if (jlo1 .lt. 1) jlo1 = 1
                 if (jlo1 .gt. nhpts) jlo1 = nhpts-1    
                 ck = (h2pts(i)-hpts(jlo1))/(hpts(jlo1+1)-hpts(jlo1))
	             optk2(i,j,k) = (one-ck)*optkg(jlo1,j,k,kkk)+ck*optkg(jlo1+1,j,k,kkk)
	             if (optk2(i,j,k) .lt. 1.0D+00) optk2(i,j,k) = 1.0D+00
                 if (optk2(i,j,k) .gt. 2.0D+00) optk2(i,j,k) = 2.0D+00
                 wgt2 = 2.0D+00 - optk2(i,j,k)
                 fthat(i,j,1,kkk) = fthat(i,j,1,kkk) + wgt2*ft(i,j,k,kkk)  
                 fthat(i,j,2,kkk) = fthat(i,j,2,kkk) + (one-wgt2)*ft(i,j,k,kkk)
              end do
            end do
          end do
                
                
           ! CONSTRUCT BEGINNING OF NEXT PERIOD DISTRIBUTION     
                
	     do i = 1,nh2pts
	        do j = 1,napts
	           do k = 1,nkpts
                call hunt(hpts,nhpts,h2pts(i),jlo1) 
                if (jlo1 .lt. 1) jlo1 = 1
                if (jlo1 .gt. nhpts) jlo1 = nhpts-1    
                    ck = (h2pts(i)-hpts(jlo1))/(hpts(jlo1+1)-hpts(jlo1))
                    gk = (one-ck)*opthg(jlo1,j,k,kkk)+ck*opthg(jlo1+1,j,k,kkk)
                    opth2(i,j,k) = gk
                    optphi2(i,j,k) = (one-ck)*optphig(jlo1,j,k,kkk)+ck*optphig(jlo1+1,j,k,kkk)
                    optc2(i,j,k) = (one-ck)*optcg(jlo1,j,k,kkk)+ck*optcg(jlo1+1,j,k,kkk)
                    opti2(i,j,k) = (one-ck)*optig(jlo1,j,k,kkk)+ck*optig(jlo1+1,j,k,kkk)

                    call hunt(h2pts,nh2pts,gk,jlo2)
                    
                if (jlo2 .lt. 1) then
                    jlo2 = 1
                    wgt1 = 1.0D+00
                elseif (jlo2 .ge. nh2pts) then
                    jlo2 = nh2pts-1
                    wgt1 = 0.0D+00
                else   
                    wgt1 = one-(gk-h2pts(jlo2))/(h2pts(jlo2+1)-h2pts(jlo2))
                endif
	           
	             if (wgt1 .lt. 0.0D+00) then 
	               print*, 'negative wgt1'
	               print*, gk,h2pts(jlo2),h2pts(jlo2+1)-h2pts(jlo2)
	               pause
	             endif
 
                 ! intertemporal distribution (t+1)       
               do kk = 1,napts
    
	              ft(jlo2,kk,k,kkk+1) = pi(j,kk)*wgt1*fthat(i,j,k,kkk)+ft(jlo2,kk,k,kkk+1)
	              ft(jlo2+1,kk,k,kkk+1) = pi(j,kk)*(one-wgt1)*fthat(i,j,k,kkk)+ft(jlo2+1,kk,k,kkk+1)
	             
	           end do
	        end do
	     end do
	     end do
        
         
        pop1 = 0.0D+00
        pop2 = 0.0D+00
        do i = 1,nh2pts
            do j = 1,napts
            ! Correct tiny numerical errors
                if (fthat(i,j,1,kkk) .lt. 1.0D-08 .and. fthat(i,j,1,kkk) .gt. 0.0D+00) then
                   fthat(i,j,2,kkk) = fthat(i,j,2,kkk) + fthat(i,j,1,kkk)
                   fthat(i,j,1,kkk) = 0.0D+00
                endif
                pop1 = pop1 + fthat(i,j,1,kkk)
                pop2 = pop2 + fthat(i,j,2,kkk)
            end do
        end do
         
        if (pop1 .lt. 1.0D-08) pop1 = 1.0D-08
        if (pop2 .lt. 1.0D-08) pop2 = 1.0D-08
         
         pops(1,kkk) = pop1
         pops(2,kkk) = pop2
         
         fsum = sum(ft(:,:,:,kkk))
         
	     if (dabs(fsum-one) .gt. 1.0D-06) then
	        write(6,739) kkk,fsum
 739        format('integration failure in ft: ',i6,f18.10)
            pause
	     endif
	     
	     fsum = sum(fthat(:,:,:,kkk))
	     if (dabs(fsum-one) .gt. 1.0D-06) then
	        write(6,740) kkk,fsum
 740        format('integration failure in fthat: ',i6,f18.10)
            pause
	     endif
	     

        bigh1 = 0.0D+00
        bigh2 = 0.0D+00
        bigq1 = 0.0D+00
        bigq2 = 0.0D+00
        bigc1 = 0.0D+00
        bigc2 = 0.0D+00
        bigqcheck = 0.0D+00
        hp1 = 0.0D+00
        hp2 = 0.0D+00
        
        fnrm(:,:,1) = fthat(:,:,1,kkk)/pop1   ! Normalized intratemporal distributions
        fnrm(:,:,2) = fthat(:,:,2,kkk)/pop2

        do j = 1,napts
            do i = 1,nh2pts
                hp1   = hp1 + h2pts(i)*ft(i,j,1,kkk+1)
                hp2   = hp2 + h2pts(i)*ft(i,j,2,kkk+1)
                bigh1 = bigh1+h2pts(i)*fnrm(i,j,1)
                bigh2 = bigh2+h2pts(i)*fnrm(i,j,2)
                bigc1 = bigc1 + optc2(i,j,1)*fnrm(i,j,1)
                bigc2 = bigc2 + optc2(i,j,2)*fnrm(i,j,2)
                bigq1 = bigq1 + optphi2(i,j,1)*fthat(i,j,1,kkk)  
                bigq2 = bigq2 + optphi2(i,j,2)*fthat(i,j,2,kkk)  
                bigi1 = bigi1 + opti2(i,j,1)*fnrm(i,j,1)
                bigi2 = bigi2 + opti2(i,j,2)*fnrm(i,j,2)                           
            end do
        end do

        if (bigq1 .lt. 1.0d-08) then
           bigq1 = 1.0d-08
        endif
        if (bigq2 .lt. 1.0d-08) then 
           bigq2 = 1.0d-08
        endif
        
        bigq(1,kkk) = bigq1
        bigq(2,kkk) = bigq2
        bigc(1,kkk) = bigc1
        bigc(2,kkk) = bigc2
        
        hp1 = hp1/pop1
        hp2 = hp2/pop2
        
        phat(1,kkk) = (wfix/mu)*(bigq1/lfix1)**((one-mu)/mu)
        phat(2,kkk) = (wfix/mu)*(bigq2/lfix2)**((one-mu)/mu) 
        
        if (phat(1,kkk) .lt. pmin) phat(1,kkk) = pmin
        if (phat(2,kkk) .lt. pmin) phat(2,kkk) = pmin
        if (pop1 .le. 1.0D-08) phat(1,kkk) = pmin
        if (pop2 .le. 1.0D-08) phat(2,kkk) = pmin
        
        if (phat(1,kkk) .gt. 10.0D+00) phat(1,kkk) = 10.0D+00
        if (phat(2,kkk) .gt. 10.0D+00) phat(2,kkk) = 10.0D+00
        
        hhat(1,kkk) = bigh1   
        hhat(2,kkk) = bigh2 
        
        bigh1 = bigh1*pop1
        bigh2 = bigh2*pop2
        
        hp(1,kkk) = hp1
        hp(2,kkk) = hp2
      
        if (hhat(1,kkk) .lt. hpts(1)) hhat(1,kkk) = hpts(1)
        if (hhat(2,kkk) .lt. hpts(1)) hhat(2,kkk) = hpts(1)

        write(6,387) kkk,hcap(1,kkk),hhat(1,kkk),hcap(2,kkk),hhat(2,kkk),pcap(1,kkk),phat(1,kkk),pcap(2,kkk),phat(2,kkk),pops(1,kkk),pops(2,kkk)
387     format(i3,2f7.3,2f5.1,2f7.3,2f6.2,2f7.3)
        
	    if (multpl(kkk,1000) .eq. 1) then
	        write(6,256) kkk
 256        format(i5)
         endif

      end do

      return
	  end
	  
	  subroutine splint2(xa,ya,y2a,x,y,yp,ydp,khi,klo,kvals,n,ndoy,ndoyp,ndoydp,na,nk)
      use parameters_mod
      implicit none
      integer, intent(in) :: n,ndoy,ndoyp,ndoydp,kvals,na
      integer, intent(inout) :: khi,klo
      real*8, intent(in), dimension(n) :: xa
      real*8, intent(in), dimension(n,napts,nkpts) :: ya,y2a
      real*8, intent(in) :: x
      real*8, intent(out) :: y,yp,ydp
      integer :: k,nk
      real*8 :: h,a,b,asq,bsq

      if (kvals == 0) then
         klo = 1
         khi = n
 1       if ((khi-klo) > 1) then
            k = (khi+klo)/2
            if (xa(k) > x) then
               khi = k
            else
               klo = k
            endif
            goto 1
         endif
      endif

      h = xa(khi)-xa(klo)
      a = (xa(khi)-x)/h
      b = (x-xa(klo))/h
      asq = a*a
      bsq = b*b

      if (ndoy == 1) y = a*ya(klo,na,nk)+b*ya(khi,na,nk)+((asq*a-a)*y2a(klo,na,nk)+(bsq*b-b)*y2a(khi,na,nk))*(h*h)/6.0D+00
      if (ndoyp == 1) yp = (ya(khi,na,nk)-ya(klo,na,nk))/h-(3.0D+00*asq-one)/6.0D+00*h*y2a(klo,na,nk)+(3.0D+00*bsq-one)/6.0D+00*h*y2a(khi,na,nk)
      if (ndoydp == 1) ydp = a*y2a(klo,na,nk)+b*y2a(khi,na,nk)
      
      return
      end
      
      subroutine interp(xx,v,vdp)
      use parameters_mod
      implicit none
      real*8, dimension(nhpts), intent(in) :: xx
      real*8, dimension(nhpts) :: a,b,c,r,yy
      real*8, dimension(nhpts,napts,2), intent(in) :: v
      real*8, dimension(nhpts,napts,2), intent(out) :: vdp
      integer :: i,j,k,multpl,nk
     
      a(1) = 0.0D+00
      a(nhpts) = -1.0D+00
      b(1) = 1.0D+00
      b(nhpts) = 1.05D+00
      c(1) = -1.05D+00
      c(nhpts) = 0.0D+00
      r(1) = 0.0D+00
      r(nhpts) = 0.0D+00
      
      do k = 1,nkpts
         nk = k
      do j = 1,napts
         do i = 2,nhpts-1
            a(i) = (xx(i)-xx(i-1))/6.0D+00
            b(i) = (xx(i+1)-xx(i-1))/3.0D+00
            c(i) = (xx(i+1)-xx(i))/6.0D+00
            r(i) = (v(i+1,j,nk)-v(i,j,nk))/(xx(i+1)-xx(i))-(v(i,j,nk)-v(i-1,j,nk))/(xx(i)-xx(i-1))
         end do
         call tridag(a,b,c,r,vdp(:,j,nk),nhpts)
      end do
      end do
    
      return
      end
      
      subroutine tridag(a,b,c,r,u,n)
      implicit none
      real*8, parameter :: toler = 1.0D-12
      integer, parameter :: nmax = 50000
      integer, intent(in) :: n
      real*8, dimension(n) :: a,b,c,r,u
      real*8, dimension(nmax) :: gam
      real*8 :: bet
      integer :: j

      bet = b(1)
      u(1) = r(1)/bet
      do j = 2,n
         gam(j) = c(j-1)/bet
         bet = b(j)-a(j)*gam(j)
         if (dabs(bet) < toler) then
            write(6,100)
 100        format('failure in tridag')
            pause
         endif
         u(j) = (r(j)-a(j)*u(j-1))/bet
      end do
      do j = n-1,1,-1
         u(j) = u(j)-gam(j+1)*u(j+1)
      end do

      return
      end
      
      integer function multpl(i,m)  
      implicit none
      integer, intent(in) :: i,m
      real*8, parameter :: toler = 1.0D-12
      real*8 :: remain

      remain = dble(real(i))/dble(real(m))-i/m

      if (remain .le. toler) then
         multpl = 1
      else
         multpl = 0
      endif

      return
      end
      
      subroutine hunt(xx,n,x,jlo)
      implicit none
      integer, intent(in) :: n
      real*8, intent(in), dimension(n) :: xx
      logical :: ascnd
      real*8,  intent(in) :: x
      integer, intent(inout) :: jlo
      integer :: jhi,inc,jm

      ascnd = xx(n) .ge. xx(1)
      if (jlo .le. 0 .or. jlo .gt. n) then
         jlo = 0
         jhi = n+1
         goto 3
      endif

      inc = 1
      
      if (x .ge. xx(jlo) .eqv. ascnd) then
 1       jhi = jlo+inc
         if (jhi .gt. n) then
            jhi = n+1
         elseif (x .ge. xx(jhi) .eqv. ascnd) then
            jlo = jhi
            inc = inc+inc
            goto 1
         endif
      else
         jhi = jlo
 2       jlo = jhi-inc
         if (jlo .lt. 1) then
            jlo = 0
         elseif (x .lt. xx(jlo) .eqv. ascnd) then
            jhi = jlo
            inc = inc+inc
            goto 2
         endif
      endif
 3    if (jhi-jlo .eq. 1) then
         if (x .eq. xx(n)) jlo = n-1
         if (x .eq. xx(1)) jlo = 1
         return
      endif
      jm = (jhi+jlo)/2
      if (x .ge. xx(jm) .eqv. ascnd) then
         jlo = jm
      else
         jhi = jm
      endif
      goto 3

      end
                                
      double precision function getxhp(inv,hum,ab,nbd)
      use parameters_mod
      use states_mod
      implicit none
      real*8 :: inv,hum,ab,elas,sh_a,sh_i,sh_H,biga,gam
      integer :: nbd
      
      gam = gam1
      
      if (nbd .eq. 1) then
        if (bigaswitch .eq. 1) then
           biga = biga2 
        else
           biga = biga1 
        endif
      else
        biga = biga2
      endif

      sh_a = lam
      sh_i = lam
      sh_H = lam
      
      if (inv .lt. 1.0D-08) then
        getxhp = 1.0D-08
      else       
          getxhp = biga*(sh_a*ab**gam+sh_i*inv**gam+sh_H*hum**gam)**(one/gam)
      endif
      
      return
      end
      
      double precision function getdxhp(inv,hum,ab,nbd)
      use parameters_mod
      use states_mod
      implicit none
      real*8 :: inv,hum,ab,elas,bigf,sh_a,sh_i,sh_H,biga,gam
      integer :: nbd
      
      gam = gam1
      
      if (nbd .eq. 1) then
        if (bigaswitch .eq. 1) then
           biga = biga2
        else
           biga = biga1
        endif
      else
        biga = biga2
      endif

      sh_a = lam
      sh_i = lam
      sh_H = lam
      
      bigf = biga*(sh_a*ab**(gam)+sh_i*inv**gam+sh_H*hum**gam)**((one/gam)-one)
      getdxhp = bigf*one*sh_i*inv**(gam-one)
      
      return
      end
      

	  subroutine func(np,j,x,fj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: fj
      real*8 :: xhp,c,it,phi,utilc,yval,yp,ydp,ab,humstay,getxhp,p
      integer :: khi,klo
 
      it = x(1)
      c = x(2)
      p = pcap(nbdstay,nt)
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))
      
      humstay = hcap(nbdstay,nt)
      ab = apts(nap)
      xhp = getxhp(it,humstay,ab,nbdstay)
      
      
      if ((nuc .eq. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = dlog(c)+theta*dlog(phi)
         
      elseif ((nuc .ne. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = (c**(one-nuc))/(one-nuc)+theta*dlog(phi)
         
      elseif ((nuc .eq. 1.0D+00) .and. (nus .ne. 1.0D+00)) then
         utilc = dlog(c)+theta*(phi**(one-nus))/(one-nus)
         
      else
         utilc = (c**(one-nuc))/(one-nuc)+theta*(phi**(one-nus))/(one-nus) 
           
      endif
 
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,1,0,0,nap,nbdstay)
       
      fj = -(utilc+beta*yval)
      
      return
      end

	  subroutine func2(np,j,x,fj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: fj
      real*8 :: xhp,c,it,phi,utilc,yval,yp,ydp,ab,humgo,getxhp,p
      integer :: klo,khi
 
      it = x(1)
      c = x(2)
      p = pcap(nbdgo,nt)
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))
      
      humgo = hcap(nbdgo,nt)
      ab = apts(nap)
      xhp = getxhp(it,humgo,ab,nbdgo)
      
      
      if ((nuc .eq. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = dlog(c)+theta*dlog(phi)
         
      elseif ((nuc .ne. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = (c**(one-nuc))/(one-nuc)+theta*dlog(phi)
         
      elseif ((nuc .eq. 1.0D+00) .and. (nus .ne. 1.0D+00)) then
         utilc = dlog(c)+theta*(phi**(one-nus))/(one-nus)
         
      else
         utilc = (c**(one-nuc))/(one-nuc)+theta*(phi**(one-nus))/(one-nus)   
         
      endif

      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,1,0,0,nap,nbdgo)
      
      fj = -(utilc+beta*yval)
     
      return
      end

      subroutine dfunc(np,j,x,fj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: fj
      real*8 :: xhp,c,it,yval,phi,yp,ydp,ab,humstay,getxhp,dhpdi,getdxhp,dphidc,p
      integer :: klo,khi
      external dummy
      
      it = x(1)
      c = x(2)
      p = pcap(nbdstay,nt)
      phi =(theta/p)**(one/nus)*(c**(nuc/nus))
     
      humstay = hcap(nbdstay,nt)
      ab = apts(nap)
      xhp = getxhp(it,humstay,ab,nbdstay)
      dhpdi = getdxhp(it,humstay,ab,nbdstay)
      dphidc = (nuc/nus)*((theta/p)**(one/nus))*c**((nuc/nus)-one)
      
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,0,1,0,nap,nbdstay)
  
      fj(1) = -beta*yp*dhpdi
      fj(2) = -c**(-nuc)-theta*dphidc*phi**(-nus)
      
      return
      end

      subroutine dfunc2(np,j,x,fj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: fj
      real*8 :: xhp,c,it,yval,yp,ydp,ab,humgo,getxhp,dhpdi,getdxhp,p,phi,dphidc
      integer :: klo,khi
      external dummy
      
      it = x(1)
      c = x(2)
      p = pcap(nbdgo,nt)
      phi =(theta/p)**(one/nus)*(c**(nuc/nus))
      
      humgo = hcap(nbdgo,nt)
      ab = apts(nap)
      xhp = getxhp(it,humgo,ab,nbdgo)
      
      dhpdi = getdxhp(it,humgo,ab,nbdgo)
      dphidc = (nuc/nus)*((theta/p)**(one/nus))*c**((nuc/nus)-one)
      
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,0,1,0,nap,nbdgo)
  
      fj(1) = -beta*yp*dhpdi
      fj(2) = -c**(-nuc)-theta*dphidc*phi**(-nus)
      
      return
      end
    
      subroutine const(np,j,x,gj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: gj
      real*8 :: xhp,c,it,phi,p
      integer :: khi,klo
      
      it = x(1)
      c = x(2)
      p = pcap(nbdstay,nt)
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))

      gj = c+it+p*phi-wage*xhcur
        
      return
      end
      
      
      subroutine const2(np,j,x,gj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: gj
      real*8 :: xhp,c,it,phi,p
      integer :: khi,klo
      
      it = x(1)
      c = x(2)
      p = pcap(nbdgo,nt)
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))
       
      gj = c+it+p*phi-wage*xhcur+costm
      
      return
      end
                
      subroutine dconst(np,j,x,gj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: gj
      real*8 :: c,p,phi,dphidc
      external dummy
      
      c = x(2)
      p = pcap(nbdstay,nt)
      
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))
      dphidc = (nuc/nus)*((theta/p)**(one/nus))*c**((nuc/nus)-one)
      
      gj(1) = one
      gj(2) = one+p*dphidc

      return
      end
      
      subroutine dconst2(np,j,x,gj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: gj
      real*8 :: c,p,phi,dphidc
      external dummy
      
      c = x(2)
      p = pcap(nbdgo,nt)
      
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))
      dphidc = (nuc/nus)*((theta/p)**(one/nus))*c**((nuc/nus)-one)
      
      
      gj(1) = one
      gj(2) = one+p*dphidc
      
      return
      end
                                
      double precision function zbrent2(funct,x1,x2,price,tol)
      implicit none
      real*8 :: x1,x2,tol,zip,fa,fb,a,b,c,fc,d,e,tol1,xm,s,p,q,r,price
      integer :: iter
      real*8, parameter :: eps = 3.0D-08
      integer, parameter :: itmax = 300 
      interface
         double precision function funct(x,pr)
         implicit none
         real*8, intent(in) :: x,pr
         end function funct
      end interface

      a = x1
      b = x2
      fa = funct(a,price)
      fb = funct(b,price)
      zip = 0.0D+00

      if ((fa > zip .and. fb > zip) .or. (fa < zip .and. fb < zip)) then
      print*,'root must be bracketed'
      print*, a,b
      print*, fa,fb
      endif

      c = b
      fc = fb
      do iter = 1,itmax
        if (iter .eq. itmax) print*, 'HIT ITMAX'
         if ((fb > zip .and. fc > zip) .or. (fb < zip .and. fc < zip)) then
            c = a
            fc = fa
            d = b-a
            e = d
         endif
         if (dabs(fc) < dabs(fb)) then
            a = b
            b = c
            c = a
            fa = fb
            fb = fc
            fc = fa
         endif
         tol1 = 2.0D+00*eps*dabs(b)+0.5D+00*tol
         xm = 0.5D+00*(c-b)
         if (dabs(xm) <= tol1 .or. fb == zip) then
            zbrent2 = b
            return
         endif
         if (dabs(e) >= tol1 .and. dabs(fa) > dabs(fb)) then
            s = fb/fa
            if (a == c) then
               p = 2.0D+00*xm*s
               q = 1.0D+00-s
            else
               q = fa/fc
               r = fb/fc
               p = s*(2.0D+00*xm*q*(q-r)-(b-a)*(r-1.0D+00))
               q = (q-1.0D+00)*(r-1.0D+00)*(s-1.0D+00)
            endif
            if (p > zip) q = -q
            p = dabs(p)
            if (2.0D+00*p < dmin1(3.0D+00*xm*q-dabs(tol1*q),dabs(e*q))) then
               e = d
               d = p/q
            else
               d = xm
               e = d
            endif
         else
            d = xm
            e = d
         endif
         a = b
         fa = fb
         if (dabs(d) > tol1) then
            b = b+d
         else
            b = b+sign(tol1,xm)
         endif
         fb = funct(b,price)
      end do

      pause 'zbrent2 exceeding maximum iterations'
      zbrent2 = b

      return
      end
      
      double precision function zbrent(funct,x1,x2,tol)
      implicit none
      real*8 :: x1,x2,tol,zip,fa,fb,a,b,c,fc,d,e,tol1,xm,s,p,q,r,price
      integer :: iter
      real*8, parameter :: eps = 3.0D-08
      integer, parameter :: itmax = 300 
      interface
         double precision function funct(x)
         implicit none
         real*8, intent(in) :: x
         end function funct
      end interface

      a = x1
      b = x2
      fa = funct(a)
      fb = funct(b)
      zip = 0.0D+00

      if ((fa > zip .and. fb > zip) .or. (fa < zip .and. fb < zip)) then
      print*,'root must be bracketed'
      print*, a,b
      print*, fa,fb
      endif

      c = b
      fc = fb
      do iter = 1,itmax
        if (iter .eq. itmax) print*, 'HIT ITMAX'
         if ((fb > zip .and. fc > zip) .or. (fb < zip .and. fc < zip)) then
            c = a
            fc = fa
            d = b-a
            e = d
         endif
         if (dabs(fc) < dabs(fb)) then
            a = b
            b = c
            c = a
            fa = fb
            fb = fc
            fc = fa
         endif
         tol1 = 2.0D+00*eps*dabs(b)+0.5D+00*tol
         xm = 0.5D+00*(c-b)
         if (dabs(xm) <= tol1 .or. fb == zip) then
            zbrent = b
            return
         endif
         if (dabs(e) >= tol1 .and. dabs(fa) > dabs(fb)) then
            s = fb/fa
            if (a == c) then
               p = 2.0D+00*xm*s
               q = 1.0D+00-s
            else
               q = fa/fc
               r = fb/fc
               p = s*(2.0D+00*xm*q*(q-r)-(b-a)*(r-1.0D+00))
               q = (q-1.0D+00)*(r-1.0D+00)*(s-1.0D+00)
            endif
            if (p > zip) q = -q
            p = dabs(p)
            if (2.0D+00*p < dmin1(3.0D+00*xm*q-dabs(tol1*q),dabs(e*q))) then
               e = d
               d = p/q
            else
               d = xm
               e = d
            endif
         else
            d = xm
            e = d
         endif
         a = b
         fa = fb
         if (dabs(d) > tol1) then
            b = b+d
         else
            b = b+sign(tol1,xm)
         endif
         fb = funct(b)
      end do

      pause 'zbrent exceeding maximum iterations'
      zbrent = b

      return
      end
      
      double precision function upperb(x,p)
	  use parameters_mod
	  use states_mod
	  implicit none
	  real*8 :: phi
	  real*8, intent(in) :: x,p

      phi = (theta/p)**(one/nus)*(x**(nuc/nus))
	  upperb = p*phi+x-wfix*xhcur

	  return
	  end
	  
	  double precision function welfare(wel)
      use parameters_mod
      use states_mod
      implicit none
      real*8, intent(in) :: wel
      integer :: jl
      real*8 :: cl,term
      
      jl = 1
      call hunt(lpts,nlpts,wel,jl)
      if (jl < 1) jl = 1
      if (jl >= nlpts) jl = nlpts-1
      cl = dabs((wel-lpts(jl))/(lpts(jl+1)-lpts(jl)))
      term = (one-cl)*vhat2(nh,nap,nk,jl)+cl*vhat2(nh,nap,nk,jl+1)
    
      welfare = term-vtrans


      return 
      end