
          ! Use to find final steady state
!**********************************************************************************************************************************
!PARAMETERS
!**********************************************************************************************************************************
      module parameters_mod
      implicit none
     
      integer, parameter :: nhpts = 150
      integer, parameter :: nh2pts = 10000
      integer, parameter :: napts = 9
      integer, parameter :: nkpts = 2
      integer, parameter :: nread = 1
      integer, parameter :: nread2 = 0
      integer, parameter :: nread3 = 1
      integer, parameter :: mxiter = 50
      integer, parameter :: mxiter1 = 20  ! iterations on p
      integer, parameter :: mxiter2 = 1000  ! iterations on H
      integer, parameter :: nlzpts = 500
      integer, parameter :: iwsize = 50
      integer, parameter :: nwsize = 500
      integer, parameter :: nparam = 2
      integer, parameter :: nf = 1
      integer, parameter :: nineq = 1
      integer, parameter :: nineqn = 1
      integer, parameter :: neq = 0      
      integer, parameter :: neqn = 0
      integer, parameter :: ncal = 4
      integer, parameter :: bigaswitch = 1  ! 0 - Wilson Case, 1 - MLK Case
      real*8, parameter :: costm = 1000000.0D+00  ! Set to a very large number for X case
   
      real*8, parameter :: one = 1.0D+00
      real*8, parameter :: biga2 = 5.0D+00 
      real*8, parameter :: hhom = 0.250D+00   
      real*8, parameter :: phom = 0.500D+00  
      real*8, parameter :: mu = 0.64D+00   
      real*8, parameter :: beta = 0.665D+00
      real*8, parameter :: nuc = 1.5D+00
      real*8, parameter :: nus = 2.0D+00
      real*8, parameter :: wfix = 1.00D+00
      real*8, parameter :: pmin = 1.0D-06
      real*8, parameter :: lam = one/3.0D+00
      real*8, parameter :: lfix1 = 0.114D+00
      real*8, parameter :: lfix2 = one-lfix1
      
               
      real*8, dimension(napts,napts) :: pi
      
      end module parameters_mod

!**********************************************************************************************************************************
!DECISION RULES, etc
!**********************************************************************************************************************************      
      module states_mod
        use parameters_mod
        implicit none
        integer :: nbdstay,nbdgo
        real*8 :: xhcur,wage,bigy,wcap,what,theta,gam1,biga1,pstay,pgo
        real*8, dimension(nhpts,napts,nkpts) :: y2,opti2,opth2,optc2,optk2,optphi2
        real*8, dimension(nh2pts,napts,nkpts) :: f0,f1,f1hat
        real*8, dimension(nhpts,napts,nkpts) :: v,vdp
        real*8, dimension(nkpts) :: hcap,pcap,hhat,phat,pophat,bighum,pop,bigc,bigq,bigi
        
        integer :: naps
        
        real*8, dimension(nhpts) :: hpts
        real*8, dimension(nhpts,2) :: upbndstay,upbndgo
        real*8, dimension(nh2pts) :: h2pts
        real*8, dimension(napts) :: apts    
        real*8, dimension(nkpts) :: kpts
      end module states_mod
  
!**********************************************************************************************************************************
!MAIN PROGRAM
!**********************************************************************************************************************************    

     
      program main
        use parameters_mod
        use states_mod
      implicit none
      integer :: i,j,k,kkk
      real*8 :: dum,rho,siga,hinc,update
      real*8, dimension(ncal) :: xcal
   
!***********************************************************     
!Build human capital grids
!***********************************************************  
      open(2,file='hgrid.in',form='formatted')
        do i = 1,nhpts
            read(2,458) hpts(i)
458         format(f18.10)
        end do
      close(2)
      
      h2pts(1) = hpts(1)
      hinc = (hpts(nhpts)-hpts(1))/dble(real(nh2pts-1))
      do i = 2,nh2pts
         h2pts(i) = h2pts(i-1)+hinc
      end do

      open(3,file='nbd_calib.best',form='formatted')
      do i = 1,ncal
         read(3,132) xcal(i),dum
 132     format(2f18.10)
      end do
      close(3)
      
!***********************************************************
!Build ability process
!***********************************************************
      
      biga1  = xcal(1)
      siga   = xcal(2)
      theta = xcal(3)
      gam1   = xcal(4)
      
      rho = 0.0D+00
      
      apts(1) = -sqrt(dble(napts-1))*siga/sqrt(one-rho*rho)
      apts(napts) = -apts(1)
      do i = 2,napts-1
         apts(i) = apts(i-1)+(apts(napts)-apts(1))/dble(napts-1)
      end do
      apts = dexp(apts)
      
      call rouwenhorst(rho,pi,napts)
      
      kpts(1) = one
      kpts(2) = one + one
!***********************************************************     
! Value function and decision rules
!***********************************************************
      if (nread .eq. 1) then
         open(2,file='saving_fin.out',form='formatted')
         do k = 1,nkpts
         do j = 1,napts
         do i = 1,nhpts
           write(3,456) dum,dum,dum,y2(i,j,k),dum,opth2(i,j,k),opti2(i,j,k),optc2(i,j,k),optphi2(i,j,k),optk2(i,j,k),dum,dum
456         format(2f12.6,f5.1,6f18.10,f5.1,f18.10,i4)
         end do
         end do 
         end do
         close(2)
      else
         do i = 1,nhpts
            y2(i,:,:) = dlog(hpts(i)+1.0D-01)
            optc2(i,:,:) = 0.5D+00*hpts(i)   
            opti2(i,:,:) = 0.5D+00*hpts(i)
         end do
      endif

!***********************************************************     
! Distribution of human capital
!***********************************************************
      if (nread2 .eq. 1) then
        open(2,file='saving_fin.f1',form='formatted')
         do k = 1,nkpts
            do i = 1,nh2pts
            read(2,547) dum,dum,f1(i,:,k)
 547        format(11f18.10)          
            end do
         end do
       close(2)
      else
         f1 = 0.0D+00
         f1(300,2,1) = 0.114D+00
         f1(300,2,2) = one-0.114D+00
      endif
      
      wcap = wfix
      
      if (nread3 .eq. 1) then 
        open(2,file='saving_fin.agg',form='formatted')
        do k = 1,nkpts
            read(2,902) dum,dum,hcap(k),dum,pcap(k),dum,dum,dum,pop(k)
 902        format(i4,9f16.10)
        end do
        close(2)
      else
      hcap(1) = 13.2687614436D+00 
      hcap(2) = 23.2962446630D+00    
      pcap(1) = 1.6322239208D+00
      pcap(2) = 2.0140150370D+00    
      pop(1) = 0.114D+00
      pop(2) = 0.886D+00
      endif 
      
      if (costm .gt. 500.0D+00) then
          hcap(1) = hcap(2)
          pcap(1) = pcap(2)
      endif
       
      print*, 'getting started'  
      do j = 1,mxiter2
       print*, 'prices', pcap  
      do k = 1,mxiter1
      
        call zeta2

        print*, 'price diff', pcap-phat

        if (maxval(dabs(pcap-phat)) .lt. 5.0D-04) goto 13
        pcap = (one-phom)*pcap+phom*phat
        
      end do

13     if (maxval(dabs(hcap-hhat)) .lt. 5.0D-02) goto 18
       print*, 'human cap', hcap
       print*, 'human cap diff', hcap-hhat
        
       hcap = (one-hhom)*hcap+hhom*hhat
       print*, 'H updated'


        print*, 'population', pop(1),pop(2)     
        print*, j,k
        print*, '--------------------'
       
       end do
       
18      print*, 'hcap =', hcap
        print*, 'price =', pcap
        print*, 'pop=', pop

        
      print*, 'done'
      pause
      end
      
!!!!!!!!!!!!!!!!!!!!!!!!END OF MAIN PROGRAM !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1      
      
      subroutine zeta2
      use parameters_mod
      use states_mod 
      implicit none
      
      integer :: i,j,k,kkk,multpl,mode,iprint,miter,inform,isave
      
      integer :: khi,klo,ierr,nk
      integer, dimension(nhpts,napts,nkpts) :: information
      
      real*8 :: vmxdif,dmxdif,cmxdif,bmxdif,amxdif,eps,epsneq,udelta,bigbnd, &
                kstay,knot,ck,cstay,cswitch,istay,ystay,hstay,phistay, &
                humstay,humgo,getxhp,littlehouse,kgo,ygo,igo,cgo,phigo
      real*8 :: zbrent
      real*8, dimension(nhpts,napts,nkpts) :: y,optk,optc,opti,optphi,budget,yalt,optystay,optymove,optimove,optcmove,optistay,optcstay,opthstay,opthmove,optphimove,optphistay
      real*8, dimension(nh2pts,napts,nkpts) :: optig,opthg,optcg,optkg,optphig
      real*8, dimension(nwsize) :: work
      real*8, dimension(nparam) :: x,bl,bu,bcheck,bl2,bu2,bcheck2
      real*8, dimension(nf+1) :: ff
      real*8, dimension(nineq+neq+1) :: g
      real*8, dimension(2) :: upboundn
      real*8, dimension(3) :: xx2,xx3
      integer, dimension(nwsize) :: iw

      interface 
	     subroutine funca(np,j,x,fj)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out) :: fj
	     end subroutine funca
	  end interface
	  interface 
	     subroutine dfunc(np,j,x,fj,dummy)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out), dimension(np) :: fj
	        external dummy
	     end subroutine dfunc
	  end interface
	  interface 
	     subroutine const(np,j,x,gj)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out) :: gj
         end subroutine const
	  end interface
      interface
         subroutine dconst(np,j,x,gj,dummy)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out), dimension(np) :: gj
	        external dummy
	     end subroutine dconst
	  end interface
	  interface 
	     subroutine funca2(np,j,x,fj)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out) :: fj
	     end subroutine funca2
	  end interface
	  interface 
	     subroutine dfunc2(np,j,x,fj,dummy)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out), dimension(np) :: fj
	        external dummy
	     end subroutine dfunc2
	  end interface
	  interface 
	     subroutine const2(np,j,x,gj)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out) :: gj
         end subroutine const2
	  end interface
      interface
         subroutine dconst2(np,j,x,gj,dummy)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out), dimension(np) :: gj
	        external dummy
	     end subroutine dconst2
	  end interface
	  external grobfd,grcnfd

      interface
         double precision function upperbstay(x)
            real*8, intent(in) :: x
         end function upperbstay
      end interface
      interface
         double precision function upperbgo(x)
            real*8, intent(in) :: x
         end function upperbgo
      end interface
      
      mode = 100
      iprint = 1000
      eps = 1.0D-12
      miter = 500
      epsneq = 1.0D-12
      udelta = 0.0D+00
	  bigbnd = 1.0D+10
	  isave = 0

      wage = wcap
      	 
	  vmxdif = 100.0D+00

	  do kkk = 1,mxiter
             
         y = y2
         opti = opti2
         optc = optc2
         optk = optk2
         optphi = optphi2

              v = 0.0D+00
              do i = 1,napts
                 do j = 1,napts
                   do k = 1,nkpts
                    v(:,i,k) = v(:,i,k)+pi(i,j)*y(:,j,k)
                   end do
                end do
              end do
             
              call interp(hpts,v,vdp)
             
         do k = 1,nkpts
            nk = k

            littlehouse = 1.0D-08
 
            if (nk .eq. 1) then
               kstay = 1.0D+00
               kgo   = 2.0D+00 
               nbdstay = 1
               nbdgo = 2        
               humstay = hcap(1)
               humgo   = hcap(2)        
               pstay = pcap(1)
               pgo = pcap(2)  
            elseif (nk .eq. 2) then
               kstay = 2.0D+00
               kgo   = 1.0D+00
               nbdstay = 2
               nbdgo = 1   
               humstay = hcap(2)
               humgo   = hcap(1)
               pstay = pcap(2)
               pgo   = pcap(1)
            endif     
               
         do i = 1,nhpts
           xhcur = hpts(i)
           upbndstay(i,k) = zbrent(upperbstay,1.0D-08,wcap*xhcur,1.0D-08) 
           upbndgo(i,k) = zbrent(upperbgo,1.0D-08,wcap*xhcur,1.0D-08)
         end do
      
            do j = 1,napts
           
                naps = j
              
                do i = 1,nhpts

                xhcur = hpts(i)
                
                bl(1) = 1.0D-08
                bl(2) = 1.0D-08
                
                bu(1) = wage*xhcur*0.9999D+00-pstay*littlehouse
                bu(2) = upbndstay(i,k)
                
                bcheck = bu-bl
                
                if (minval(bcheck) .lt. 1.0D-08) then
                   print*, 'error on constraint set for staying'
                   print*, bl
                   print*, bu
                   print*, xhcur,pstay
                   pause
                endif
                
                if (i .eq. 1) then
                  x(1) = opti(i,j,k)
                  x(2) = optc(i,j,k)
                else
                  x(1) = optistay(i,j,k)
                  x(2) = optcstay(i,j,k)
                endif
                 
                call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
                          epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,funca,const,dfunc,dconst)
                !
                !call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, & 
                !          epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,funca,const,grobfd,dconst)   !grobfd,grcnfd      
                   
                istay = x(1)
                cstay = x(2)
                phistay = (theta/pstay)**(one/nus)*(cstay**(nuc/nus))
                ystay = -ff(1)
                optistay(i,j,k) = istay
                optcstay(i,j,k) = cstay
                optphistay(i,j,k) = phistay
                optystay(i,j,k) = ystay
                
                bl2(1) = 1.0D-08
                bl2(2) = 1.0D-08
                
                bu2(1) = wage*xhcur*0.9999D+00-pgo*littlehouse-costm
                bu2(2) = upbndgo(i,k)-costm
                
                bcheck = bu2-bl2
                
              if (minval(bcheck) .le. 1.0D-08) then 
                   cgo = 1.0D-08
                   igo = 1.0D-08
                   phigo = (theta/pgo)**(one/nus)*(cgo**(nuc/nus))
                   ygo = -1.0D+06
                   opthmove(i,j,k) = hpts(1)
              else           
                if (i .eq. 1) then
                  x(1) = opti(i,j,k)
                  x(2) = optc(i,j,k)
                else
                  x(1) = optimove(i,j,k)
                  x(2) = optcmove(i,j,k)
                endif
                  
                  call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, & 
                          epsneq,udelta,bl2,bu2,x,ff,g,iw,iwsize,work,nwsize,funca2,const2,dfunc2,dconst2)   !grobfd,grcnfd  
                   
                    
                   igo = x(1)
                   cgo = x(2)
                   phigo = (theta/pgo)**(one/nus)*(cgo**(nuc/nus))
                   ygo = -ff(1)
                   optimove(i,j,k) = igo
                   optcmove(i,j,k) = cgo
                   optphimove(i,j,k) = phigo
                   optymove(i,j,k) = ygo
                   
                   opthstay(i,j,k) = getxhp(istay,humstay,apts(naps),nbdstay)
                   opthmove(i,j,k) = getxhp(igo,humgo,apts(naps),nbdgo)

                  endif
                  if(ystay .gt. ygo) then
                      opth2(i,j,k) = getxhp(istay,humstay,apts(naps),nbdstay)
                      optc2(i,j,k) = cstay
                      opti2(i,j,k) = istay
                      optphi2(i,j,k) = phistay
                        
                      optk2(i,j,k) = kstay
                      y2(i,j,k) = ystay 
                      yalt(i,j,k) = ygo
                      if (yalt(i,j,k) .lt. -1.0D+05) then 
                         yalt(i,j,k) = 0.0D+00
                      endif     
                      budget(i,j,k) = istay+cstay+pstay*phistay-wage*xhcur
                      information(i,j,k) = inform  
                   else            
                      opth2(i,j,k) = getxhp(igo,humgo,apts(naps),nbdgo)
                      optc2(i,j,k) = cgo
                      opti2(i,j,k) = igo
                      optphi2(i,j,k) = phigo
                      optk2(i,j,k) = kgo
                      y2(i,j,k) = ygo 
                      yalt(i,j,k) = ystay 
                      if (yalt(i,j,k) .lt. -1.0D+05) then 
                         yalt(i,j,k) = 0.0D+00
                      endif          
                      budget(i,j,k) = igo+cgo+pgo*phigo-wage*xhcur
                      information(i,j,k) = inform   
                   endif                    
                end do
            end do
         end do
         
 2       vmxdif = maxval(dabs(y2-y))
         amxdif = maxval(dabs(opti2-opti))
         bmxdif = maxval(dabs(optc2-optc))
         dmxdif = maxval(dabs(optk2-optk))
         
         xx2 = maxloc(dabs(y2-y))
         xx3 = maxloc(dabs(opti2-opti))

         if (vmxdif .lt. 1.0D-05 .and. kkk .gt. 2) goto 8

         if (multpl(kkk,1) .eq. 1) then
            write(6,888) kkk,mxiter,vmxdif,amxdif,bmxdif,dmxdif
 888        format(2i7,4f12.6)
         endif

      end do
      
 8    continue
 
     open(3,file='saving_fin.out',form='formatted')
        do k = 1,nkpts
          do j = 1,napts
            do i = 1,nhpts
            write(3,456) hpts(i),apts(j),kpts(k),y2(i,j,k),yalt(i,j,k),opth2(i,j,k),opti2(i,j,k),optc2(i,j,k),optphi2(i,j,k),optk2(i,j,k),budget(i,j,k),information(i,j,k)
456         format(2f12.6,f5.1,6f18.10,f5.1,f18.10,i4)
            end do
         end do
      end do
      close(3)
      
      open(3,file='compare.out',form='formatted')
      do k = 1,nkpts
          do j = 1,napts                
            do i = 1,nhpts
            write(3,457) hpts(i),apts(j),kpts(k),optk2(i,j,k),optystay(i,j,k),optymove(i,j,k),opthstay(i,j,k),opthmove(i,j,k),optistay(i,j,k),optimove(i,j,k),optcstay(i,j,k),optcmove(i,j,k),optphistay(i,j,k),optphimove(i,j,k)
 457        format(2f12.6,2f5.1,10f18.10)
            end do
         end do
      end do
      close(3)
 
    
!      write(6,7984)
! 7984 format('computing invariant distribution')

      call calcf

      open(22,file='saving_fin.f1',form='formatted')
      do k = 1,nkpts
         do i = 1,nh2pts
            write(22,599) h2pts(i),kpts(k),f1(i,:,k)
 599        format(11f18.10)
         end do
      end do
      close(22)
      
      open(22,file='saving_fin.fhat',form='formatted')
      do i = 1,nh2pts
         do k = 1,nkpts
         write(22,599) h2pts(i),kpts(k),f1hat(i,:,k)
         end do
      end do
      close(22)
      
      open(22,file='saving_fin.agg',form='formatted')
        do k = 1,nkpts
            write(22,902) k,bighum(k),hcap(k),hhat(k),pcap(k),phat(k),bigi(k),bigc(k),bigq(k),pop(k)
 902        format(i4,9f16.10)
        end do
      close(22)
      
      return
      end

      subroutine splint2(xa,ya,y2a,x,y,yp,ydp,khi,klo,kvals,n,ndoy,ndoyp,ndoydp,na,nk)
      use parameters_mod
      implicit none
      integer, intent(in) :: n,ndoy,ndoyp,ndoydp,kvals,na
      integer, intent(inout) :: khi,klo
      real*8, intent(in), dimension(n) :: xa
      real*8, intent(in), dimension(n,napts,nkpts) :: ya,y2a
      real*8, intent(in) :: x
      real*8, intent(out) :: y,yp,ydp
      integer :: k,nk
      real*8 :: h,a,b,asq,bsq

      if (kvals == 0) then
         klo = 1
         khi = n
 1       if ((khi-klo) > 1) then
            k = (khi+klo)/2
            if (xa(k) > x) then
               khi = k
            else
               klo = k
            endif
            goto 1
         endif
      endif

      h = xa(khi)-xa(klo)
      a = (xa(khi)-x)/h
      b = (x-xa(klo))/h
      asq = a*a
      bsq = b*b

      if (ndoy == 1) y = a*ya(klo,na,nk)+b*ya(khi,na,nk)+((asq*a-a)*y2a(klo,na,nk)+(bsq*b-b)*y2a(khi,na,nk))*(h*h)/6.0D+00
      if (ndoyp == 1) yp = (ya(khi,na,nk)-ya(klo,na,nk))/h-(3.0D+00*asq-one)/6.0D+00*h*y2a(klo,na,nk)+(3.0D+00*bsq-one)/6.0D+00*h*y2a(khi,na,nk)
      if (ndoydp == 1) ydp = a*y2a(klo,na,nk)+b*y2a(khi,na,nk)
      
      return
      end
      
      subroutine interp(xx,v,vdp)
      use parameters_mod
      implicit none
      real*8, dimension(nhpts), intent(in) :: xx
      real*8, dimension(nhpts) :: a,b,c,r,yy
      real*8, dimension(nhpts,napts,2), intent(in) :: v
      real*8, dimension(nhpts,napts,2), intent(out) :: vdp
      integer :: i,j,k,multpl,nk
     
      a(1) = 0.0D+00
      a(nhpts) = -1.0D+00
      b(1) = 1.0D+00
      b(nhpts) = 1.05D+00
      c(1) = -1.05D+00
      c(nhpts) = 0.0D+00
      r(1) = 0.0D+00
      r(nhpts) = 0.0D+00
      
      do k = 1,nkpts
         nk = k
      do j = 1,napts
         do i = 2,nhpts-1
            a(i) = (xx(i)-xx(i-1))/6.0D+00
            b(i) = (xx(i+1)-xx(i-1))/3.0D+00
            c(i) = (xx(i+1)-xx(i))/6.0D+00
            r(i) = (v(i+1,j,nk)-v(i,j,nk))/(xx(i+1)-xx(i))-(v(i,j,nk)-v(i-1,j,nk))/(xx(i)-xx(i-1))
         end do
         call tridag(a,b,c,r,vdp(:,j,nk),nhpts)
      end do
      end do  

      return
      end

      subroutine tridag(a,b,c,r,u,n)
      implicit none
      real*8, parameter :: toler = 1.0D-12
      integer, parameter :: nmax = 50000
      integer, intent(in) :: n
      real*8, dimension(n) :: a,b,c,r,u
      real*8, dimension(nmax) :: gam
      real*8 :: bet
      integer :: j

      bet = b(1)
      u(1) = r(1)/bet
      do j = 2,n
         gam(j) = c(j-1)/bet
         bet = b(j)-a(j)*gam(j)
         if (dabs(bet) < toler) then
            write(6,100)
 100        format('failure in tridag')
            pause
         endif
         u(j) = (r(j)-a(j)*u(j-1))/bet
      end do
      do j = n-1,1,-1
         u(j) = u(j)-gam(j+1)*u(j+1)
      end do

      return
      end      

      integer function multpl(i,m)
      implicit none
      integer, intent(in) :: i,m
      real*8, parameter :: toler = 1.0D-12
      real*8 :: remain

      remain = dble(real(i))/dble(real(m))-i/m

      if (remain .le. toler) then
         multpl = 1
      else
         multpl = 0
      endif

      return
      end

      
      subroutine hunt(xx,n,x,jlo)
      implicit none
      integer, intent(in) :: n
      real*8, intent(in), dimension(n) :: xx
      logical :: ascnd
      real*8,  intent(in) :: x
      integer, intent(inout) :: jlo
      integer :: jhi,inc,jm

      ascnd = xx(n) .ge. xx(1)
      if (jlo .le. 0 .or. jlo .gt. n) then
         jlo = 0
         jhi = n+1
         goto 3
      endif

      inc = 1
      
      if (x .ge. xx(jlo) .eqv. ascnd) then
 1       jhi = jlo+inc
         if (jhi .gt. n) then
            jhi = n+1
         elseif (x .ge. xx(jhi) .eqv. ascnd) then
            jlo = jhi
            inc = inc+inc
            goto 1
         endif
      else
         jhi = jlo
 2       jlo = jhi-inc
         if (jlo .lt. 1) then
            jlo = 0
         elseif (x .lt. xx(jlo) .eqv. ascnd) then
            jhi = jlo
            inc = inc+inc
            goto 2
         endif
      endif
 3    if (jhi-jlo .eq. 1) then
         if (x .eq. xx(n)) jlo = n-1
         if (x .eq. xx(1)) jlo = 1
         return
      endif
      jm = (jhi+jlo)/2
      if (x .ge. xx(jm) .eqv. ascnd) then
         jlo = jm
      else
         jhi = jm
      endif
      goto 3

      end
         
      subroutine calcf
      use parameters_mod
      use states_mod
      implicit none
      integer, parameter :: mxit = 500
      real*8, dimension(nh2pts,napts,nkpts) :: opthg,optig,optcg,optphig,optkg,vgrid
      real*8 :: hinc,wght,fmxdif,fsum,bigh1,bigc1,bigi1,wght1,wght2,pop1,pop2,bigq1,ck
      real*8 :: dum
      integer :: i,j,k,kk,kkk,jk,jlo,khi,klo,multpl
      integer :: xcheck,xcheck1,xcheck2,nct,jlo1
      integer, dimension(3) :: xxx
      real*8, dimension(nh2pts,napts,nkpts) :: wghts1,wghts2
      integer, dimension(nh2pts,napts,nkpts) :: jlos1
  
      open(4,file='saving.fst',form='formatted')
      do k = 1,nkpts
         do i = 1,nh2pts
            read(4,546) dum,dum,f1(i,:,k)
 546        format(11f18.10)           
         end do
       end do
       close(4)
       
	  hinc = h2pts(2)-h2pts(1)
      
      jk = 1
          
      do k = 1,nkpts
         do j = 1,napts
            do i = 1,nh2pts
                call hunt(hpts,nhpts,h2pts(i),jk)
            if (jk .lt. 1) jk = 1
            if (jk .ge. nhpts) jk = nhpts-1
                ck = (h2pts(i)-hpts(jk))/(hpts(jk+1)-hpts(jk))
                opthg(i,j,k) = (one-ck)*opth2(jk,j,k)+ck*opth2(jk+1,j,k)
                optig(i,j,k) = (one-ck)*opti2(jk,j,k)+ck*opti2(jk+1,j,k)
                optcg(i,j,k) = (one-ck)*optc2(jk,j,k)+ck*optc2(jk+1,j,k)
                optphig(i,j,k) = (one-ck)*optphi2(jk,j,k)+ck*optphi2(jk+1,j,k)
                optkg(i,j,k) = (one-ck)*optk2(jk,j,k) + ck*optk2(jk+1,j,k)
                if (optkg(i,j,k) .lt. 1.0D+00) then
                    optkg(i,j,k) = 1.0D+00
                endif
                if (optkg(i,j,k) .gt. 2.0D+00) then
                    optkg(i,j,k) = 2.0D+00
                endif

                wght2 = 2.0D+00-optkg(i,j,k)
	             
	            if (wght2 .gt. 1.0D+00) then 
	               print*, 'wght2 > 1' 
	               print*, wght2,optkg(i,j,k),ck
	               pause
	            endif
	            if (wght2 .lt. 0.0D+00) then
	               print*, 'wght2 < 1' 
	               print*, wght2,optkg(i,j,k),ck
	               pause
	            endif 
                
                wghts2(i,j,k) = wght2
                
                vgrid(i,j,k) = (one-ck)*y2(jk,j,k)+ck*y2(jk+1,j,k)
                
            end do
         end do
      end do
   
      jlo1 = 1 
 
      do i = 1,nh2pts
         do j = 1,napts
            do k = 1,nkpts
                call hunt(h2pts,nh2pts,opthg(i,j,k),jlo1)      
                if (jlo1 .lt. 1) then
                    jlo1 = 1
                    wght1 = 1.0D+00
                elseif (jlo1 .ge. nh2pts) then
                    jlo1 = nh2pts-1
                    wght1 = 0.0D+00
                else   
                    wght1 = one-(opthg(i,j,k)-h2pts(jlo1))/(h2pts(jlo1+1)-h2pts(jlo1))
                endif
                
                wghts1(i,j,k) = wght1
	            jlos1(i,j,k) = jlo1

	        end do
	     end do
      end do

      do kkk = 1,mxit
         
         f0 = f1

	     f1 = 0.0D+00
	     f1hat = 0.0D+00
!     Find the intratemporal distubution
         do i = 1,nh2pts
            do j = 1,napts
               do k = 1,nkpts
                  wght2 = wghts2(i,j,k)
                  f1hat(i,j,1) = wght2*f0(i,j,k)+f1hat(i,j,1)
	              f1hat(i,j,2) = (one-wght2)*f0(i,j,k)+f1hat(i,j,2)
	            end do
	        end do
	     end do

         do i = 1,nh2pts
            do j = 1,napts
               do k = 1,nkpts
                  jlo1 = jlos1(i,j,k)
		          wght1 = wghts1(i,j,k)
	              
		            do kk = 1,napts
		            ! pi is the fraction moving from ability j to ability kk
		            ! wght1 is the fraction going to h2pt(jlo1)
		            ! wght2 is the fraction going to community 1  
                      f1(jlo1,kk,k) = pi(j,kk)*wght1*f1hat(i,j,k)+f1(jlo1,kk,k)   ! eps to eps' and going to nbd1 at xhlo 
                      f1(jlo1+1,kk,k) = pi(j,kk)*(one-wght1)*f1hat(i,j,k)+f1(jlo1+1,kk,k) ! eps to eps' and going to nbd1 at xhhi                    
                    end do
               end do
            end do
        end do
        
    
         fmxdif = maxval(dabs(f1-f0))
         xxx = maxloc(dabs(f1-f0))
         fsum = sum(f1)

         if (dabs(fsum-one) .gt. 1.0D-06) then
            write(6,543) kkk,fsum
 543        format('does not integrate to 1: ',i10,f18.10)
            pause
         endif
         if (multpl(kkk,100) .eq. 1) then
            write(6,703) kkk,mxit,fmxdif
            print*, xxx
            print*, fsum
 703        format(2i10,f18.10)
         endif
         if (fmxdif .le. 1.0D-08) goto 1
         
      end do
     
 1      pop1 = 0.0D+00
        pop2 = 0.0D+00

        do i = 1,nh2pts
            do j = 1,napts
                pop1 = pop1 + f1hat(i,j,1)
                pop2 = pop2 + f1hat(i,j,2)
            end do
        end do
        
        pop(1) = pop1
        pop(2) = pop2
        print*, 'pop', pop
         
        do k = 1,nkpts

        bigh1 = 0.0D+00
        bigq1 = 0.0D+00
        bigc1 = 0.0D+00
        bigi1 = 0.0D+00
        
        do j = 1,napts
            do i = 1,nh2pts
                bigh1 = bigh1+h2pts(i)*f1hat(i,j,k)
                bigq1 = bigq1 + optphig(i,j,k)*f1hat(i,j,k)
                bigc1 = bigc1 + optcg(i,j,k)*f1hat(i,j,k)
                bigi1 = bigi1 + optig(i,j,k)*f1hat(i,j,k)
            end do
        end do
         
        if (bigh1 .lt. hpts(1)) then
          hhat(k) = hpts(1)
        else 
          hhat(k) = bigh1/pop(k)
        endif
        
        bighum(k) = bigh1
        
        bigq(k) = bigq1
        bigc(k) = bigc1
        bigi(k) = bigi1
     end do

        bigy = sum(bigi+bigc)
        
        what = wfix
          
        if (bigq(1) .lt. 1.0d-08) then
           bigq(1) = 1.0d-08
        endif
        if (bigq(2) .lt. 1.0d-08) then 
           bigq(2) = 1.0d-08
        endif

        phat(1) = (what/mu)*(bigq(1)/lfix1)**((one-mu)/mu)
        phat(2) = (what/mu)*(bigq(2)/lfix2)**((one-mu)/mu)
        
        if (phat(1) .lt. pmin) phat(1) = pmin
        if (phat(2) .lt. pmin) phat(2) = pmin
        if (pop(1) .lt. 1.0D-08) phat(1) = pmin
        if (pop(2) .lt. 1.0D-08) phat(2) = pmin
      

      return
      end


	   subroutine funca(np,j,x,fj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: fj
      real*8 :: xhp,c,it,phi,utilc,yval,yp,ydp,ab,humstay,getxhp
      integer :: khi,klo,ierr
      logical :: skip
      
      skip = .true.
      
      it = x(1)
      c = x(2)
      phi = (theta/pstay)**(one/nus)*(c**(nuc/nus))
     
      humstay = hcap(nbdstay)
      ab = apts(naps)
      xhp = getxhp(it,humstay,ab,nbdstay)
      
      
      if ((nuc .eq. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = dlog(c)+theta*dlog(phi)
         
      elseif ((nuc .ne. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = (c**(one-nuc))/(one-nuc)+theta*dlog(phi)
         
      elseif ((nuc .eq. 1.0D+00) .and. (nus .ne. 1.0D+00)) then
         utilc = dlog(c)+theta*(phi**(one-nus))/(one-nus)
         
      else
         utilc = (c**(one-nuc))/(one-nuc)+theta*(phi**(one-nus))/(one-nus) 
           
      endif
      
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,1,0,0,naps,nbdstay)
      
      fj = -(utilc+beta*yval)

      return
      end


      subroutine funca2(np,j,x,fj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: fj
      real*8 :: xhp,c,it,phi,utilc,yval,yp,ydp,ab,humgo,getxhp
      integer :: khi,klo,ierr
      logical :: skip
      
      skip = .true.
      
      it = x(1)
      c = x(2)
      phi = (theta/pgo)**(one/nus)*(c**(nuc/nus))

      ab = apts(naps)
      humgo = hcap(nbdgo)
      xhp = getxhp(it,humgo,ab,nbdgo)
     
      if ((nuc .eq. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = dlog(c)+theta*dlog(phi)
         
      elseif ((nuc .ne. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = (c**(one-nuc))/(one-nuc)+theta*dlog(phi)
         
      elseif ((nuc .eq. 1.0D+00) .and. (nus .ne. 1.0D+00)) then
         utilc = dlog(c)+theta*(phi**(one-nus))/(one-nus)
         
      else
         utilc = (c**(one-nuc))/(one-nuc)+theta*(phi**(one-nus))/(one-nus) 
           
      endif
      
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,1,0,0,naps,nbdgo)
      
      fj = -(utilc+beta*yval)
      
      return
      end
      
      subroutine dfunc(np,j,x,fj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: fj
      real*8 :: xhp,c,it,yval,phi,yp,ydp,ab,humstay,getxhp,dhpdi,getdxhp,dphidc
      integer :: khi,klo
      external dummy
      
      
      it = x(1)
      c = x(2)
      phi =(theta/pstay)**(one/nus)*(c**(nuc/nus))
      
      ab = apts(naps)
      xhp = getxhp(it,hcap(nbdstay),ab,nbdstay)
      dhpdi = getdxhp(it,hcap(nbdstay),ab,nbdstay)
      dphidc = (nuc/nus)*(theta/pstay)**(one/nus)*c**((nuc/nus)-one)
      
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,0,1,0,naps,nbdstay)

      fj(1) = -beta*yp*dhpdi
      fj(2) = -c**(-nuc)-theta*dphidc*phi**(-nus)
      
      return
      end
      
      
      subroutine dfunc2(np,j,x,fj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: fj
      real*8 :: xhp,c,it,yval,phi,yp,ydp,ab,humstay,getxhp,dhpdi,getdxhp,dphidc
      integer :: khi,klo
      external dummy
      
      
      it = x(1)
      c = x(2)
      phi =(theta/pgo)**(one/nus)*(c**(nuc/nus))
      
      ab = apts(naps)
      xhp = getxhp(it,hcap(nbdgo),ab,nbdgo)
      dhpdi = getdxhp(it,hcap(nbdgo),ab,nbdgo)
      dphidc = (nuc/nus)*(theta/pgo)**(one/nus)*c**((nuc/nus)-one)
      
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,0,1,0,naps,nbdgo)


      fj(1) = -beta*yp*dhpdi
      fj(2) = -c**(-nuc)-theta*dphidc*phi**(-nus)
      
      return
      end
      

      subroutine const(np,j,x,gj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: gj
      real*8 :: xhp,c,it,phi
      integer :: khi,klo
      
      it = x(1)
      c = x(2)
      phi = (theta/pstay)**(one/nus)*(c**(nuc/nus))
       
      gj = c+it+pstay*phi-wage*xhcur

      return
      end
      
      subroutine const2(np,j,x,gj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: gj
      real*8 :: xhp,c,it,phi
      integer :: khi,klo
      
      it = x(1)
      c = x(2)
      phi = (theta/pgo)**(one/nus)*(c**(nuc/nus))
      
      gj = c+it+pgo*phi-wage*xhcur+costm
 
      return
      end
      
                
      subroutine dconst(np,j,x,gj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: gj
      real*8 :: c,phi,dphidc
      external dummy
      
      c = x(2)
      
      phi =(theta/pstay)**(one/nus)*(c**(nuc/nus))
      dphidc = (nuc/nus)*(theta/pstay)**(one/nus)*c**((nuc/nus)-one)
      
      gj(1) = one
      gj(2) = one+pstay*dphidc
      
      return
      end
      
      subroutine dconst2(np,j,x,gj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: gj
      real*8 :: c,phi,dphidc
      external dummy
      
      c = x(2)
      
      phi =(theta/pgo)**(one/nus)*(c**(nuc/nus))
      dphidc = (nuc/nus)*((theta/pgo)**(one/nus))*c**((nuc/nus)-one)
      
      gj(1) = one
      gj(2) = one+pgo*dphidc
   
      return
      end
      
      double precision function getxhp(inv,hum,ab,nk)
      use parameters_mod
      use states_mod
      implicit none
      real*8 :: inv,hum,ab,elas,exter,pers,sh_a,sh_i,sh_H,biga,gam
      integer :: nk
      
      gam = gam1 
      
      if (nk .eq. 1) then
        if (bigaswitch .eq. 1) then 
           biga = biga2 
        else
           biga = biga1 
        endif
      else
        biga = biga2
      endif
  
      sh_a = lam
      sh_i = lam
      sh_H = lam
    
      if (inv .lt. 1.0D-08) then
         getxhp = 1.0D-08
      else 
          getxhp = biga*(sh_a*ab**gam+sh_i*inv**gam+sh_H*hum**gam)**(one/gam)
      endif
      
      return
      end
      
      double precision function getdxhp(inv,hum,ab,nk)
      use parameters_mod
      use states_mod
      implicit none
      real*8 :: inv,hum,ab,elas,bigf,bigz,sh_a,sh_i,sh_H,biga,gam
      integer :: nk
      
      gam = gam1
      
      if (nk .eq. 1) then
        if (bigaswitch .eq. 1) then
           biga = biga2 
        else
           biga = biga1 
        endif
      else
        biga = biga2
      endif
      
      sh_a = lam
      sh_i = lam
      sh_H = lam

       bigf = biga*(sh_a*ab**(gam)+sh_i*inv**gam+sh_H*hum**gam)**((one/gam)-one)
       getdxhp = bigf*one*sh_i*inv**(gam-one)
       
      return
      end
      
      double precision function zbrent(func,x1,x2,tol)
      implicit none
      real*8 :: x1,x2,tol,zip,fa,fb,a,b,c,fc,d,e,tol1,xm,s,p,q,r,zz
      integer :: iter
      real*8, parameter :: eps = 3.0D-08
      integer, parameter :: itmax = 300 
      interface
         double precision function func(x)
         implicit none
         real*8, intent(in) :: x
         end function func
      end interface

      a = x1
      b = x2
      fa = func(a)
      fb = func(b)
      zip = 0.0D+00

      if ((fa > zip .and. fb > zip) .or. (fa < zip .and. fb < zip)) then
      print*,'root must be bracketed'
      print*, a,b
      print*, fa,fb
      endif

      c = b
      fc = fb
      do iter = 1,itmax
        if (iter .eq. itmax) print*, 'HIT ITMAX'
         if ((fb > zip .and. fc > zip) .or. (fb < zip .and. fc < zip)) then
            c = a
            fc = fa
            d = b-a
            e = d
         endif
         if (dabs(fc) < dabs(fb)) then
            a = b
            b = c
            c = a
            fa = fb
            fb = fc
            fc = fa
         endif
         tol1 = 2.0D+00*eps*dabs(b)+0.5D+00*tol
         xm = 0.5D+00*(c-b)
         if (dabs(xm) <= tol1 .or. fb == zip) then
            zbrent = b
            return
         endif
         if (dabs(e) >= tol1 .and. dabs(fa) > dabs(fb)) then
            s = fb/fa
            if (a == c) then
               p = 2.0D+00*xm*s
               q = 1.0D+00-s
            else
               q = fa/fc
               r = fb/fc
               p = s*(2.0D+00*xm*q*(q-r)-(b-a)*(r-1.0D+00))
               q = (q-1.0D+00)*(r-1.0D+00)*(s-1.0D+00)
            endif
            if (p > zip) q = -q
            p = dabs(p)
            if (2.0D+00*p < dmin1(3.0D+00*xm*q-dabs(tol1*q),dabs(e*q))) then
               e = d
               d = p/q
            else
               d = xm
               e = d
            endif
         else
            d = xm
            e = d
         endif
         a = b
         fa = fb
         if (dabs(d) > tol1) then
            b = b+d
         else
            b = b+sign(tol1,xm)
         endif
         fb = func(b)
      end do

      pause 'zbrent exceeding maximum iterations'
      zbrent = b

      return
      end
      
	double precision function upperbstay(x)
	use parameters_mod
	use states_mod
	implicit none
	real*8, intent(in) :: x
	real*8 :: phi
	  
	phi = (theta/pstay)**(one/nus)*(x**(nuc/nus))
	upperbstay = pstay*phi+x-wcap*xhcur

	return
	end 
	
	double precision function upperbgo(x)
	use parameters_mod
	use states_mod
	implicit none
	real*8, intent(in) :: x
	real*8 :: phi
	  
	phi = (theta/pgo)**(one/nus)*(x**(nuc/nus))
	upperbgo = pgo*phi+x-wcap*xhcur

	return
	end