
   !  Use to get calibration of 1960 initial steady state
   
!**********************************************************************************************************************************
!PARAMETERS
!**********************************************************************************************************************************
      module parameters_mod
      implicit none
     
      integer, parameter :: nhpts = 150 
      integer, parameter :: hadd = 9
      integer, parameter :: nh2pts = 10000
      integer, parameter :: nkpts = 2
      integer, parameter :: napts = 9
      integer, parameter :: onetimecal = 1 ! Takes the calibration values and runs once for good fit (Turn off and set tolerance lower for faster search over parameter space).
      integer, parameter :: nread = 0   !read in value func and decision rules
      integer, parameter :: nread2 = 0  !read in distribution
      integer, parameter :: nread3 = 1  !read in aggregates
      integer, parameter :: mxiter = 50
      integer, parameter :: mxiter2 = 500
      integer, parameter :: mxpol = 200
      integer, parameter :: nlzpts = 500
      integer, parameter :: iwsize = 50
      integer, parameter :: nwsize = 500
      integer, parameter :: nparam = 2
      integer, parameter :: nf = 1
      integer, parameter :: nineq = 1
      integer, parameter :: nineqn = 1
      integer, parameter :: neq = 0      
      integer, parameter :: neqn = 0
      integer, parameter :: ncal = 4
      integer, parameter :: nsim = 100000
      integer, parameter :: ndrop = 1000
  
      real*8, parameter :: one = 1.0D+00
      real*8, parameter :: biga2 = 5.0D+00 
      real*8, parameter :: mu = 0.64D+00   
      real*8, parameter :: pop1 = 0.114D+00
      real*8, parameter :: pop2 = one-pop1
      real*8, parameter :: beta = 0.665D+00
      real*8, parameter :: nuc = 1.5D+00
      real*8, parameter :: nus = 2.0D+00
      real*8, parameter :: wfix = 1.00D+00
      real*8, parameter :: pmin = 1.0D-06
      real*8, parameter :: lam  = 0.33333333D+00
      real*8, parameter :: hhom = 0.500D+00 
      real*8, parameter :: phom = 0.500D+00
      real*8, parameter :: whom = 0.500D+00
      real*8, parameter :: htol = 1.0D-06
      real*8, parameter :: ptol = 1.0D-06

              
      real*8, dimension(napts,napts) :: pi
      end module parameters_mod
      

!**********************************************************************************************************************************
!DECISION RULES, etc
!**********************************************************************************************************************************      
      module states_mod
        use parameters_mod
        implicit none
        real*8 :: xhcur,pstay,pgo,wage
        real*8 :: ratlow,ratbigh,corrys,ratvarh,rat1,rat2,rat3,s2c,rathouse
        real*8  :: wcap,what
        real*8, dimension(nhpts,napts,nkpts) :: y2,opti2,opth2,optk2,optc2,optphi2
        real*8, dimension(nhpts) :: hpts
        real*8, dimension(nhpts,nkpts) :: upbnd
        real*8, dimension(nh2pts) :: h2pts
        real*8, dimension(napts) :: apts
        real*8, dimension(nkpts) :: kpts
        real*8, dimension(nh2pts,napts,nkpts) :: f0,f1,vgrid,opthg
        real*8, dimension(nhpts,napts,nkpts) :: v,vdp
        real*8, dimension(nkpts) :: hcap,pcap,hhat,phat
        real*8, dimension(nkpts) :: bighum,bigc,bigi,bigq,meana,pop        
        
        integer, parameter, dimension(1) :: nseed = (/30192/)
        integer, dimension(nsim) :: apath   
        integer :: nk,naps
        
      end module states_mod
      
      module targets_mod
        use parameters_mod
        implicit none
        real*8 :: biga1,gam1,siga,rho,fbest,corrwh,theta,varlogh,rat7525_1,rat7525_2,rat7550_1,rat7550_2,rat9010_1,rat9010_2,s75_25,rat5010
      end module targets_mod
          
!**********************************************************************************************************************************
!MAIN PROGRAM
!**********************************************************************************************************************************    
    
      program main
        use parameters_mod
        use states_mod
        use targets_mod
      implicit none
      integer :: niter,i,j,k,kkk
      real*8 :: dum,hinc,zbrent
      real*8, dimension(ncal+1):: simp(ncal+1,ncal),y(ncal+1),tdum(ncal+1)
      real*8, dimension(nsim) :: rr
      logical :: nbest
      interface
         double precision function zeta1(x)
            real*8, intent(in), dimension(4) :: x ! Change when ncal changes
         end function zeta1
      end interface
                
!***********************************************************     
!Build human capital grids
!***********************************************************
      
     open(5,file='hgrid.in',form='formatted')
     do i = 1,nhpts
        read(5,920) hpts(i)
920     format(f18.10)
     end do
     close(5)
     
      pop(1) = pop1
      pop(2) = one-pop1
      
      wcap = wfix
   
     if (nread3 .eq. 1) then 
        open(2,file='saving_init.agg',form='formatted')
        do k = 1,nkpts
            read(2,902) dum,dum,hcap(k),dum,pcap(k),dum,dum,dum,dum,dum
 902        format(i4,9f16.10)
        end do
        close(2)
      else
         hcap(1) = 6.85D+00  
         hcap(2) = 11.35D+00
         pcap(1) = 0.52D+00
         pcap(2) = 1.46D+00
      endif 

      h2pts(1) = hpts(1)
      hinc = (hpts(nhpts)-hpts(1))/dble(real(nh2pts-1))
      do i = 2,nh2pts
         h2pts(i) = h2pts(i-1)+hinc
      end do

      kpts(1) = one
      kpts(2) = 2.0D+00
        !***********************************************************     
! Value function and decision rules
!***********************************************************
      if (nread .eq. 1) then 
        open(2,file='saving.out',form='formatted')
        do k = 1,nkpts
          do j = 1,napts
            do i = 1,nhpts
                    read(2,392) dum,dum,dum,y2(i,j,k),opth2(i,j,k),opti2(i,j,k),optc2(i,j,k),optphi2(i,j,k),optk2(i,j,k),dum,dum
392                 format(2f12.6,f5.1,5f18.10,f5.1,f18.10,i4)           
                end do  
            end do
        end do
        close(2)
      else
         do i = 1,nhpts
            do j = 1,napts
               do k = 1,nkpts
              y2(i,j,k) = 1.0D-01+dlog(hpts(i))
              optc2(i,j,k) = 0.2D+00*hpts(i)   
              opti2(i,j,k) = 0.4D+00*hpts(i)
              optk2(i,j,k) = kpts(k)
              optphi2(i,j,k) = 0.2D+00*hpts(i)
               end do
            end do
         end do
      endif
    
!***********************************************************     
! Distribution of human capital
!***********************************************************
      if (nread2 .eq. 1) then
         open(2,file='saving.f1',form='formatted')
         do k = 1,nkpts
            do i = 1,nh2pts
              read(2,547) dum,dum,f1(i,:,k)
 547          format(11f18.10)           
            end do
         end do
         close(2)
      else
         f1 = 0.0D+00
         f1(2500,2,1) = pop1
         f1(2500,2,2) = one-pop1
      endif  
      
!***********************************************************     
! Joint human capital stocks
!***********************************************************

      nbest = .true.
      if(nbest) then
        open(2,file='nbd_calib.best',form='formatted')
        do i = 1,ncal
           read(2,330) simp(1,i),tdum(i)
330        format(2f18.10)
        enddo    
        close(2)
        open(3,file='nbdbest.out',form='formatted') 
          read(3,340) fbest
340       format(f18.10)
        close(3)
        fbest = 0.8D+00
      else      
  
!                               INITIAL GUESS FOR SIMPLEX                                             
      
        simp(1,1) =  12.5D+00          ! bigA1  = x(1) ; ratbigh - 0.566D+00  ratio of means of human capital
        simp(1,2) =  0.614467564D+00  ! rho   = x(2) ; intergenerational correlation of log wage income
        simp(1,3) =  0.100D+00         ! theta = x(3) ; rathouse - 0.221D+00
        simp(1,4) = -0.20D+00          ! gam1    = x(4) ; 90th/10th in nbd1

        fbest = 999.0D+00
      endif
       
       do i =1,ncal 
          simp(i+1,:) = simp(1,:)
          simp(i+1,i) = (one+0.02D+00)*simp(i+1,i)
       end do
          
       
      do i = 1,ncal+1
         y(i) = zeta1(simp(i,:))  ! send parameter guess to nbd_ss code
         print*, i,y(i)
         if (onetimecal .eq. 1) then 
            print*, 'done and done'
            pause
         endif
      end do
          
      print*, 'going to amoeba'
         
      call amoeba(simp,y,ncal+1,ncal,ncal,1.0D-04,zeta1,niter)
      
      print*, 'all done'
        do i = 1,ncal+1
            print*, i,simp(1,i),y(i)
        end do
      pause
           
      end
      
!!!!!!!!!!!!!!!!!!!!!!!!END OF MAIN PROGRAM !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1      
      
      double precision function zeta1(x)
      !! Finds market clearing price given H1 and H2
      use parameters_mod
      use states_mod
      use targets_mod
      implicit none
      real*8, intent(in), dimension(ncal) :: x
      real*8, dimension(ncal) :: fv
      integer :: i,k,maxiter2
      real*8 :: zbrent,dum

      maxiter2 = 100

      biga1  = x(1)  
      siga = x(2)
      theta = x(3)
      gam1   = x(4) 
      
      if (biga1 .lt. 0.1D+00 .or. biga1 .gt. biga2 & 
             .or. siga .lt. 1.0D-04  & 
             .or. theta .lt. 0.0D+00) then
               zeta1 = 1.0D+06
               return
      endif
      
      rho = 0.0D+00
      
      apts(1) = -sqrt(dble(napts-1))*siga/sqrt(one-rho*rho)
      apts(napts) = -apts(1)
      do i = 2,napts-1
         apts(i) = apts(i-1)+(apts(napts)-apts(1))/dble(napts-1)
      end do
      
      apts = exp(apts)
 
      call rouwenhorst(rho,pi,napts)
 
      open(4,file='pi.out',form='formatted')
      do i = 1,napts
         write(4,587) pi(i,:)
587      format(9f18.10)
      end do
      close(4)     
         
      write(6,932) x
932   format(4f8.3)  ! change when ncal changes

      call simpath

      do i = 1,maxiter2

        call zeta2
        hcap = (one-hhom)*hcap+hhom*hhat
        pcap = (one-phom)*pcap+phom*phat
        print*, '--------------------'
        print*, 'human cap', hcap
        print*, 'human cap diff', hcap-hhat
        print*, 'prices', pcap
        print*, 'price diff', pcap-phat
        print*, '--------------------'
        if (maxval(dabs(hcap-hhat)) .lt. htol .and. maxval(dabs(pcap-phat)) .lt. ptol) goto 13
      end do
      
13      print*, 'hcap =', hcap
        print*, 'price =', pcap

        print*, 'simulating'
      
        call simh
     
      fv(1) =  (ratbigh - 0.563D+00)/0.563D+00   ! H1/H2 
      fv(2) =  (corrwh - 0.40D+00)/0.40D+00    ! intergenerational correlation of log earnings  
      fv(3) =  (rathouse - 0.221D+00)/0.221D+00  ! ratio of housing services to non-housing consumption
      fv(4) =  (rat7550_1 - 1.1830D+00)/1.183D+00  !75/25 in nbd1

      print*, 'fv check'
      print*, '      parameter        stat          target           error'  
      write(6,732) biga1,ratbigh,0.563D+00,fv(1)    ! ratio of means of H
      write(6,732) siga,corrwh,0.40D+00,fv(2)       ! intergenerational correlation of log earnings in nbd 2
      write(6,732) theta,rathouse,0.221D+00,fv(3)  ! ratio of housing services to non-housing consumption
      write(6,732) gam1,rat7550_1,1.1830D+00,fv(4)      ! income ratio of 75th/50th in nbd 1 1.183
      write(6,732) gam1,rat7550_2,1.1672D+00,(rat7550_2 - 1.1672D+00)/1.1672D+00 ! income ratio of 75th/50th in nbd 1 1.1672
732   format(4f15.5)
      
      zeta1 = 0.0D+00
      
      do i = 1,ncal
         print*, i,fv(i)
         zeta1 = zeta1+fv(i)*fv(i)
      end do
      
      if (zeta1 < fbest) then
         fbest = zeta1
         open(1,file='nbdbest.out',form='formatted')
         write(1,900) fbest
 900     format(f18.10)        
         close(1)
         
         open(1,file='nbd_calib.best',form='formatted')
         do i = 1,ncal
             write(1,901) x(i), fv(i)
         end do
 901     format(2f18.10)
         close(1)
      endif
      
      write(6,592) zeta1,fbest,maxval(dabs(fv)),maxloc(fv)
 592  format('deviations: ',3f18.10,i4)

        
      end

!**********************************************************************************************************************************
!Subroutines and Functions
!**********************************************************************************************************************************  
      
      subroutine zeta2
      use parameters_mod
      use states_mod 
      use targets_mod
      implicit none
      
      integer :: i,j,k,ii,kk,kkk,jk,multpl,mode,iprint,miter,inform,npol,na,isave
      
      integer :: khi,klo,ierr
      integer, dimension(nhpts,napts,nkpts) :: information
      
      real*8 :: vmxdif,emxdif,dmxdif,cmxdif,bmxdif,amxdif,eps,epsneq,udelta,bigbnd, &
                kstay,knot,ck,cstay,cswitch,istay,iswitch,hstay,hswitch,ystay,yswitch, &
                nstay,nswitch,phistay,phiswitch,pguess,poff, &
                humstay,humgo,getxhp,littlehousestay,littlehousego,zbrent
      real*8 :: yval,yp,ydp,win
      real*8, dimension(nkpts) :: abar,hbar,hbarother,price
      real*8, dimension(nhpts,napts) :: yn
      real*8, dimension(nhpts,napts,nkpts) :: y,optk,optc,opti,optphi,budget,y2p,differ
      real*8, dimension(nh2pts,napts,nkpts) :: optig,optcg,optkg,optphig
      real*8, dimension(nhpts) :: ym
      real*8, dimension(nwsize) :: work
      real*8, dimension(nparam) :: x,bl,bu,bcheck
      real*8, dimension(nf+1) :: ff
      real*8, dimension(nineq+neq+1) :: g
      real*8, dimension(2) :: upboundn
      real*8, dimension(3) :: xx2,xx3
      integer, dimension(nwsize) :: iw

      interface 
	     subroutine funca(np,j,x,fj)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out) :: fj
	     end subroutine funca
	  end interface
	  interface 
	     subroutine dfunc(np,j,x,fj,dummy)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out), dimension(np) :: fj
	        external dummy
	     end subroutine dfunc
	  end interface
	  interface 
	     subroutine const(np,j,x,gj)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out) :: gj
         end subroutine const
	  end interface
      interface
         subroutine dconst(np,j,x,gj,dummy)
	        integer, intent(in) :: np,j
	        real*8, intent(in), dimension(np) :: x
	        real*8, intent(out), dimension(np) :: gj
	        external dummy
	     end subroutine dconst
	  end interface
	  interface
      double precision function upperb(x)
            real*8, intent(in) :: x
         end function upperb
      end interface
	  external grobfd,grcnfd

      mode = 100
      iprint = 1000
      eps = 1.0D-12
      miter = 500
      epsneq = 1.0D-12
      udelta = 0.0D+00
	   bigbnd = 1.0D+10
	   isave = 0
	 
	   vmxdif = 100.0D+00

      wage = wcap
  
      if (nuc .eq. 1.0D+00) then
         do i = 1,nhpts
           xhcur = hpts(i)
           upbnd(i,:) = 0.9999D+00*wcap*xhcur
         end do
      else  
         do k = 1,nkpts
         pstay = pcap(k)
         do i = 1,nhpts
            xhcur = hpts(i)
            upbnd(i,k) = zbrent(upperb,1.0D-08,wcap*xhcur,1.0D-08) 
         end do
         end do
      endif 
   
      open(7,file='upperb.out',form='formatted')
      do i = 1,nhpts
      write(7,942) hpts(i),upbnd(i,:)
942   format(3f18.10)
      end do
      close(7)

	  do kkk = 1,mxiter

         y = y2
         opti = opti2
         optc = optc2
         optk = optk2
         optphi = optphi2
   
         v = 0.0D+00
           do i = 1,napts
              do j = 1,napts
                 do k = 1,nkpts
                 v(:,i,k) = v(:,i,k)+pi(i,j)*y(:,j,k)
                 end do
              end do
           end do
 
           call interp(hpts,v,vdp)

         do k = 1,nkpts
            nk = k
            
            if (nk .eq. 1) then
               kstay = 1.0D+00         
               humstay = hcap(1)        
               pstay = pcap(1)
               littlehousestay = 1.0D-08
            elseif (nk .eq. 2) then
               kstay = 2.0D+00
               humstay = hcap(2)
               pstay = pcap(2)
               littlehousestay = 1.0D-08
            endif         
            do j = 1,napts
           
                naps = j
              
                do i = 1,nhpts

                xhcur = hpts(i)
                
                bl(1) = 1.0D-08
                bl(2) = 1.0D-08
                
                bu(1) = wage*xhcur*0.9999D+00-pstay*littlehousestay
                bu(2) = upbnd(i,nk)
                
                bcheck = bu-bl
               
                if (minval(bcheck) .lt. 1.0D-08) then
                   print*, 'error on constraint set'
                   print*, bl
                   print*, bu
                   print*, wage,xhcur,pstay
                   pause
                endif
                
                if (bcheck(1) .ne. bcheck(1)) then
                   print*, 'NAN problem in 1'
                   print*, wage,xhcur,pstay      
                   pause
                endif
                
                if (bcheck(2) .ne. bcheck(2)) then
                   print*, 'NAN problem in 2'
                   print*, wage,xhcur
                   print*, theta,nuc,pstay
                   pause
                endif
                
                
                if (i .eq. 1) then
                  x(1) = opti(i,j,k)
                  x(2) = optc(i,j,k)
                else
                  x(1) = opti(i,j,k)
                  x(2) = optc(i,j,k)
                endif
                  
                call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
                          epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,funca,const,dfunc,dconst)
                
!                call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, & 
!                          epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,funca,const,grobfd,dconst)
                   
                   opti2(i,j,k) = x(1)
                   optc2(i,j,k) = x(2)
                   phistay = (theta/pstay)**(one/nus)*(x(2)**(nuc/nus))
                   optphi2(i,j,k) = phistay              
                   opth2(i,j,k) = getxhp(x(1),humstay,apts(naps),nk)
                   hstay = opth2(i,j,k)
                   optk2(i,j,k) = kstay
                   y2(i,j,k) = -ff(1)           
                   budget(i,j,k) = x(1)+x(2)+pstay*phistay-wage*xhcur
                   information(i,j,k) = inform  

                end do
            end do
         end do
         
 2       vmxdif = maxval(dabs(y2-y))
         dmxdif = maxval(dabs(optphi2-optphi))
         cmxdif = maxval(dabs(opti2-opti))
         bmxdif = maxval(dabs(optc2-optc))
         amxdif = maxval(dabs(optk2-optk))
         
         xx2 = maxloc(dabs(y2-y))
         xx3 = maxloc(dabs(opti2-opti))
         
        open(3,file='saving.out',form='formatted')
        do k = 1,nkpts
          do j = 1,napts
            do i = 1,nhpts
            write(3,456) hpts(i),apts(j),kpts(k),y2(i,j,k),opth2(i,j,k),opti2(i,j,k),optc2(i,j,k),optphi2(i,j,k),optk2(i,j,k),budget(i,j,k),information(i,j,k)
 456        format(2f12.6,f5.1,5f18.10,f5.1,f18.10,i4)
            end do
         end do
      end do
      close(3)
      
         if (vmxdif .lt. 1.0D-05 .and. kkk .gt. 2) goto 8

         if (multpl(kkk,100) .eq. 1) then
            write(6,888) kkk,mxiter,vmxdif,cmxdif,bmxdif
 888        format(2i7,3f12.6)
         endif

      end do
      
 8    continue
      
     open(3,file='saving.out',form='formatted')
        do k = 1,nkpts
          do j = 1,napts
            do i = 1,nhpts
            write(3,456) hpts(i),apts(j),kpts(k),y2(i,j,k),opth2(i,j,k),opti2(i,j,k),optc2(i,j,k),optphi2(i,j,k),optk2(i,j,k),budget(i,j,k),information(i,j,k)
            end do
         end do
      end do
      close(3)
      
    
!      write(6,7984)
! 7984 format('computing invariant distribution')

      call calcf

      open(22,file='saving.f1',form='formatted')
      do k = 1,nkpts
         do i = 1,nh2pts
            write(22,499) h2pts(i),kpts(k),f1(i,:,k)
         end do
      end do
      close(22)
      
      open(22,file='saving_init.agg',form='formatted')
        do k = 1,nkpts
            write(22,902) k,bighum(k),hcap(k),hhat(k),pcap(k),phat(k),bigi(k),bigc(k),bigq(k),pop(k)
 902        format(i4,9f16.10)
        end do
      close(22)
      
        open(22,file='saving.fst',form='formatted')
        do k = 1,nkpts
           do i = 1,nh2pts
              write(22,499) h2pts(i),kpts(k),f1(i,:,k)
499           format(11f18.10)           
           end do
        end do
         close(22)

      return
      end
      
      subroutine splint2(xa,ya,y2a,x,y,yp,ydp,khi,klo,kvals,n,ndoy,ndoyp,ndoydp,na,nk)
      use parameters_mod
      implicit none
      integer, intent(in) :: n,ndoy,ndoyp,ndoydp,kvals,na
      integer, intent(inout) :: khi,klo
      real*8, intent(in), dimension(n) :: xa
      real*8, intent(in), dimension(n,napts,nkpts) :: ya,y2a
      real*8, intent(in) :: x
      real*8, intent(out) :: y,yp,ydp
      integer :: k,nk
      real*8 :: h,a,b,asq,bsq

      if (kvals == 0) then
         klo = 1
         khi = n
 1       if ((khi-klo) > 1) then
            k = (khi+klo)/2
            if (xa(k) > x) then
               khi = k
            else
               klo = k
            endif
            goto 1
         endif
      endif

      h = xa(khi)-xa(klo)
      a = (xa(khi)-x)/h
      b = (x-xa(klo))/h
      asq = a*a
      bsq = b*b

      if (ndoy == 1) y = a*ya(klo,na,nk)+b*ya(khi,na,nk)+((asq*a-a)*y2a(klo,na,nk)+(bsq*b-b)*y2a(khi,na,nk))*(h*h)/6.0D+00
      if (ndoyp == 1) yp = (ya(khi,na,nk)-ya(klo,na,nk))/h-(3.0D+00*asq-one)/6.0D+00*h*y2a(klo,na,nk)+(3.0D+00*bsq-one)/6.0D+00*h*y2a(khi,na,nk)
      if (ndoydp == 1) ydp = a*y2a(klo,na,nk)+b*y2a(khi,na,nk)
      
      return
      end
      
      subroutine interp(xx,v,vdp)
      use parameters_mod
      implicit none
      real*8, dimension(nhpts), intent(in) :: xx
      real*8, dimension(nhpts) :: a,b,c,r
      real*8, dimension(nhpts,napts,2), intent(in) :: v
      real*8, dimension(nhpts,napts,2), intent(out) :: vdp
      integer :: i,j,k,nk
     
      a(1) = 0.0D+00
      a(nhpts) = -1.0D+00
      b(1) = 1.0D+00
      b(nhpts) = 1.05D+00
      c(1) = -1.05D+00
      c(nhpts) = 0.0D+00
      r(1) = 0.0D+00
      r(nhpts) = 0.0D+00

      do k = 1,nkpts
         nk = k
      do j = 1,napts
         do i = 2,nhpts-1
            a(i) = (xx(i)-xx(i-1))/6.0D+00
            b(i) = (xx(i+1)-xx(i-1))/3.0D+00
            c(i) = (xx(i+1)-xx(i))/6.0D+00
            r(i) = (v(i+1,j,nk)-v(i,j,nk))/(xx(i+1)-xx(i))-(v(i,j,nk)-v(i-1,j,nk))/(xx(i)-xx(i-1))
         end do
         call tridag(a,b,c,r,vdp(:,j,nk),nhpts)
      end do
      end do

      return
      end

      subroutine tridag(a,b,c,r,u,n)
      implicit none
      real*8, parameter :: toler = 1.0D-12
      integer, parameter :: nmax = 50000
      integer, intent(in) :: n
      real*8, dimension(n) :: a,b,c,r,u
      real*8, dimension(nmax) :: gam
      real*8 :: bet
      integer :: j

      bet = b(1)
      u(1) = r(1)/bet
      do j = 2,n
         gam(j) = c(j-1)/bet
         bet = b(j)-a(j)*gam(j)
         if (dabs(bet) < toler) then
            write(6,100)
 100        format('failure in tridag')
            pause
         endif
         u(j) = (r(j)-a(j)*u(j-1))/bet
      end do
      do j = n-1,1,-1
         u(j) = u(j)-gam(j+1)*u(j+1)
      end do

      return
      end      

      integer function multpl(i,m)
      implicit none
      integer, intent(in) :: i,m
      real*8, parameter :: toler = 1.0D-12
      real*8 :: remain

      remain = dble(real(i))/dble(real(m))-i/m

      if (remain .le. toler) then
         multpl = 1
      else
         multpl = 0
      endif

      return
      end

      subroutine hunt(xx,n,x,jlo)
      implicit none
      integer, intent(in) :: n
      real*8, intent(in), dimension(n) :: xx
      logical :: ascnd
      real*8,  intent(in) :: x
      integer, intent(inout) :: jlo
      integer :: jhi,inc,jm

      ascnd = xx(n) .ge. xx(1)
      if (jlo .le. 0 .or. jlo .gt. n) then
         jlo = 0
         jhi = n+1
         goto 3
      endif

      inc = 1
      
      if (x .ge. xx(jlo) .eqv. ascnd) then
 1       jhi = jlo+inc
         if (jhi .gt. n) then
            jhi = n+1
         elseif (x .ge. xx(jhi) .eqv. ascnd) then
            jlo = jhi
            inc = inc+inc
            goto 1
         endif
      else
         jhi = jlo
 2       jlo = jhi-inc
         if (jlo .lt. 1) then
            jlo = 0
         elseif (x .lt. xx(jlo) .eqv. ascnd) then
            jhi = jlo
            inc = inc+inc
            goto 2
         endif
      endif
 3    if (jhi-jlo .eq. 1) then
         if (x .eq. xx(n)) jlo = n-1
         if (x .eq. xx(1)) jlo = 1
         return
      endif
      jm = (jhi+jlo)/2
      if (x .ge. xx(jm) .eqv. ascnd) then
         jlo = jm
      else
         jhi = jm
      endif
      goto 3

      end
       
      subroutine calcf
      use parameters_mod
      use states_mod
      use targets_mod
      implicit none
      integer, parameter :: mxit = 500
      real*8, dimension(nh2pts,napts,nkpts) :: optig,optcg,optkg,optphig
      real*8 :: hinc,wght1,wght2,fmxdif,fsum,bigh1,bigq1,meana1,bigc1,bigi1,bigy,ck
      real*8 :: targ1,targ2,newtarg1,newtarg2,oldpop1,oldpop2,newpop,newpop1,newpop2,pct10_k1 &
                ,pct5_k2,varh_k1,varh_k2,vars,var_wh,covar_whs,xsum,ysum
      real*8 :: pct10_k2,pct25_k1,pct25_k2,pct50_k1,pct50_k2,pct75_k1,pct75_k2,pct90_k1,pct90_k2,pct10s_k2,pct90s_k2,pct25s_k2,pct75s_k2
      real*8 :: varlogh_k1,varlogh_k2,meanlogh_k1,meanlogh_k2
      integer :: i,j,k,kk,kkk,jk,jlo1,jlo2,icount,khi,klo,multpl
      integer :: xcheck,xcheck1,xcheck2,nct
      integer, dimension(3) :: xxx
      real*8, dimension(nh2pts,napts,nkpts) :: wghts1,wghts2,fnorm
      integer, dimension(nh2pts,napts,nkpts) :: jlos1,jlos2
      real*8, dimension(nh2pts*napts) :: ftemp2,stemp2
      real*8, dimension(nh2pts,napts) :: f21,f22
    
	   hinc = h2pts(2)-h2pts(1)

      jk = 1
      do k = 1,nkpts
         do j = 1,napts
            do i = 1,nh2pts
                call hunt(hpts,nhpts,h2pts(i),jk)
            if (jk .lt. 1) jk = 1
            if (jk .ge. nhpts) jk = nhpts-1
                ck = (h2pts(i)-hpts(jk))/(hpts(jk+1)-hpts(jk))
                opthg(i,j,k) = (one-ck)*opth2(jk,j,k)+ck*opth2(jk+1,j,k)
                optig(i,j,k) = (one-ck)*opti2(jk,j,k)+ck*opti2(jk+1,j,k)
                optcg(i,j,k) = (one-ck)*optc2(jk,j,k)+ck*optc2(jk+1,j,k)
                optphig(i,j,k) = (one-ck)*optphi2(jk,j,k)+ck*optphi2(jk+1,j,k)
                if (ck .lt. 0.5D+00) then
                    optkg(i,j,k) = optk2(jk,j,k)
                else
                    optkg(i,j,k) = optk2(jk+1,j,k)
                endif
                vgrid(i,j,k) = (one-ck)*y2(jk,j,k)+ck*y2(jk+1,j,k)
            end do
         end do
      end do
      
      jlo1 = 1
      do i = 1,nh2pts
         do j = 1,napts
            do k = 1,nkpts
                call hunt(h2pts,nh2pts,opthg(i,j,k),jlo1)
                if (jlo1 .lt. 1) then
                    jlo1 = 1
                    wght1 = 1.0D+00
                elseif (jlo1 .ge. nh2pts) then
                    jlo1 = nh2pts-1
                    wght1 = 0.0D+00
                else   
                    wght1 = one-(opthg(i,j,k)-h2pts(jlo1))/(h2pts(jlo1+1)-h2pts(jlo1))
                endif
                wghts1(i,j,k) = wght1
	            jlos1(i,j,k) = jlo1
	            
	            ck = (h2pts(i)-hpts(jk))/(hpts(jk+1)-hpts(jk))
	            if ((optkg(jlo1,j,k) .eq. 1.0D+00) .and. (optkg(jlo1+1,j,k) .eq. 1.0D+00)) then
	                wght2 = 1.0D+00
	            elseif ((optkg(jlo1,j,k) .eq. 2.0D+00) .and. (optkg(jlo1+1,j,k) .eq. 2.0D+00)) then
	                wght2 = 0.0D+00
	            elseif ((optkg(jlo1,j,k) .eq. 1.0D+00) .and. (optkg(jlo1+1,j,k) .eq. 2.0D+00)) then
	                wght2 = one-ck
	            else 
	                wght2 = ck
	            endif
	            
                wghts2(i,j,k) = wght2
	                
	        end do
	     end do
      end do
       

      do kkk = 1,mxit

         f0 = f1
	     f1 = 0.0D+00
         do i = 1,nh2pts
            do j = 1,napts
               do k = 1,nkpts
                  jlo1 = jlos1(i,j,k)
		          wght1 = wghts1(i,j,k)
		          wght2 = wghts2(i,j,k)
		            do kk = 1,napts
                      f1(jlo1,kk,1) = pi(j,kk)*wght1*wght2*f0(i,j,k)+f1(jlo1,kk,1)
                      f1(jlo1+1,kk,1) = pi(j,kk)*(one-wght1)*wght2*f0(i,j,k)+f1(jlo1+1,kk,1)
                      f1(jlo1,kk,2) = pi(j,kk)*wght1*(one-wght2)*f0(i,j,k)+f1(jlo1,kk,2)
                      f1(jlo1+1,kk,2) = pi(j,kk)*(one-wght1)*(one-wght2)*f0(i,j,k)+f1(jlo1+1,kk,2)
                    end do
               end do
            end do
        end do

         fmxdif = maxval(dabs(f1-f0))
         xxx = maxloc(dabs(f1-f0))
         fsum = sum(f1)

         if (dabs(fsum-one) .gt. 1.0D-06) then
            write(6,543) kkk,fsum
 543        format('does not integrate to 1: ',i10,f18.10)
            pause
         endif
       
         if (fmxdif .le. 1.0D-08) goto 1
         
      end do
     
 1    do k = 1,nkpts

        bigh1 = 0.0D+00
        bigq1 = 0.0D+00
        meana1 = 0.0D+00
        bigc1 = 0.0D+00
        bigi1 = 0.0D+00
        
        do j = 1,napts
            do i = 1,nh2pts   
                bigh1 = bigh1+h2pts(i)*f1(i,j,k)
                bigq1 = bigq1 + optphig(i,j,k)*f1(i,j,k)
                bigi1 = bigi1 + optig(i,j,k)*f1(i,j,k)
                meana1 = meana1 + apts(j)*f1(i,j,k)
                bigc1 = bigc1 + optcg(i,j,k)*f1(i,j,k)
            end do
        end do
        
        hhat(k) = bigh1/pop(k)
        bighum(k) = bigh1
        bigq(k) = bigq1
        bigc(k) = bigc1
        bigi(k) = bigi1
        meana(k) = meana1 
     end do


    if (bigq(1) .lt. 1.0d-08) then
        bigq(1) = 1.0d-08
    endif
    if (bigq(2) .lt. 1.0d-08) then 
        bigq(2) = 1.0d-08
    endif
        
        bigy = sum(bigc+bigi)
        
        what = wfix 
        
        phat(1) =  (what/mu)*(bigq(1)/pop1)**((one-mu)/mu)                
        phat(2) =  (what/mu)*(bigq(2)/pop2)**((one-mu)/mu)               
        
        if (phat(1) .lt. pmin) phat(1) = pmin
        if (phat(2) .lt. pmin) phat(2) = pmin
        
        rathouse = phat(2)*bigq(2)/bigc(2)
      
        f21 = f1(:,:,1)/pop1
        f22 = f1(:,:,2)/pop2
         
        targ1 = sum(f21(1,:))
        targ2 = sum(f22(1,:))
        
        xcheck1 = 0
        xcheck2 = 0

        do i = 2,nh2pts
            oldpop1 = targ1
            oldpop2 = targ2
            newpop1 = sum(f21(i,:))
            newpop2 = sum(f22(i,:))
            targ1 = targ1 + newpop1
            targ2 = targ2 + newpop2
       
            if (xcheck1 .eq. 0) then
              if (targ1 .gt. 0.10D+00) then 
                 pct10_k1 = ((0.10D+00-oldpop1)/newpop1)*h2pts(i)+((targ1-0.10D+00)/newpop1)*h2pts(i-1)
                 xcheck1 = xcheck1 + 1
              endif
            endif
            
            if (xcheck2 .eq. 0) then
              if (targ2 .gt. 0.10D+00) then 
                 pct10_k2 = ((0.10D+00-oldpop2)/newpop2)*h2pts(i)+((targ2-0.10D+00)/newpop2)*h2pts(i-1)
                 xcheck2 = xcheck2 + 1
              endif
            endif
            
            if (xcheck1 .eq. 1) then
              if (targ1 .gt. 0.25D+00) then 
                 pct25_k1 = ((0.25D+00-oldpop1)/newpop1)*h2pts(i)+((targ1-0.25D+00)/newpop1)*h2pts(i-1)
                 xcheck1 = xcheck1 + 1
              endif
            endif
            if (xcheck2 .eq. 1) then 
              if (targ2 .gt. 0.25D+00) then 
                pct25_k2 = ((0.25D+00-oldpop2)/newpop2)*h2pts(i)+((targ2-0.25D+00)/newpop2)*h2pts(i-1)
                xcheck2 = xcheck2 + 1
              endif
            endif
            
            if (xcheck1 .eq. 2) then
              if (targ1 .gt. 0.50D+00) then 
                 pct50_k1 = ((0.50D+00-oldpop1)/newpop1)*h2pts(i)+((targ1-0.50D+00)/newpop1)*h2pts(i-1)
                 xcheck1 = xcheck1 + 1
              endif
            endif
            if (xcheck2 .eq. 2) then 
              if (targ2 .gt. 0.50D+00) then 
                pct50_k2 = ((0.50D+00-oldpop2)/newpop2)*h2pts(i)+((targ2-0.50D+00)/newpop2)*h2pts(i-1)
                xcheck2 = xcheck2 + 1
              endif
            endif 
            
            if (xcheck1 .eq. 3) then
              if (targ1 .gt. 0.75D+00) then 
                 pct75_k1 = ((0.75D+00-oldpop1)/newpop1)*h2pts(i)+((targ1-0.75D+00)/newpop1)*h2pts(i-1)
                 xcheck1 = xcheck1 + 1
              endif
            endif
            if (xcheck2 .eq. 3) then 
              if (targ2 .gt. 0.75D+00) then 
                pct75_k2 = ((0.75D+00-oldpop2)/newpop2)*h2pts(i)+((targ2-0.75D+00)/newpop2)*h2pts(i-1)
                xcheck2 = xcheck2 + 1
              endif
            endif    
            
            if (xcheck1 .eq. 4) then
              if (targ1 .gt. 0.90D+00) then 
                 pct90_k1 = ((0.90D+00-oldpop1)/newpop1)*h2pts(i)+((targ1-0.90D+00)/newpop1)*h2pts(i-1)
                 xcheck1 = xcheck1 + 1
              endif
            endif
            if (xcheck2 .eq. 4) then 
              if (targ2 .gt. 0.90D+00) then 
                pct90_k2 = ((0.90D+00-oldpop2)/newpop2)*h2pts(i)+((targ2-0.90D+00)/newpop2)*h2pts(i-1)
                xcheck2 = xcheck2 + 1
              endif
            endif
            
        end do 
        
     
        varlogh_k1 = 0.0D+00
        varlogh_k2 = 0.0D+00
        
        meanlogh_k1 = 0.0D+00
        meanlogh_k2 = 0.0D+00
         
        do j = 1,napts
           do i = 1,nh2pts 
              meanlogh_k1 = meanlogh_k1+f1(i,j,1)*dlog(wage*h2pts(i))
              meanlogh_k2 = meanlogh_k2+f1(i,j,2)*dlog(wage*h2pts(i))
           end do
        end do
              
        do j = 1,napts
           do i = 1,nh2pts 
              varlogh_k1 = varlogh_k1+f1(i,j,1)*(dlog(h2pts(i))-meanlogh_k1)*(dlog(h2pts(i))-meanlogh_k1)
              varlogh_k2 = varlogh_k2+f1(i,j,2)*(dlog(h2pts(i))-meanlogh_k2)*(dlog(h2pts(i))-meanlogh_k2)
           end do
        end do
        
        varlogh = varlogh_k2
        
       nct = 1

       do i = 1,nh2pts
         do j = 1,napts
            ftemp2(nct) = f1(i,j,2)
            stemp2(nct) = optphig(i,j,2)
            nct = nct + 1
         end do
       end do

        ftemp2 = ftemp2/pop2
        
       call sort2(nh2pts*napts,stemp2,ftemp2) 
       
         xsum = ftemp2(1)

         xcheck = 0
       do i = 2,nh2pts*napts
          ysum = xsum
          newpop = ftemp2(i)
          xsum = xsum + newpop
          
          if (xcheck .eq. 0) then
             if (xsum .gt. 0.25D+00) then
                pct25s_k2 = ((0.25D+00-ysum)/newpop)*stemp2(i)+((xsum-0.25D+00)/newpop)*stemp2(i-1)  
                xcheck = 1
             endif
          endif
          
          if (xcheck .eq. 1) then
             if (xsum .gt. 0.75D+00) then
                pct75s_k2 = ((0.75D+00-ysum)/newpop)*stemp2(i)+((xsum-0.75D+00)/newpop)*stemp2(i-1)  
                xcheck = 2
             endif
          endif
          
        end do

        ratbigh = hhat(1)/hhat(2)
        rat7525_1= pct75_k1/pct25_k1
        rat7525_2= pct75_k2/pct25_k2
        rat9010_1= pct90_k1/pct10_k1
        rat9010_2= pct90_k2/pct10_k2
        rat7550_1 = pct75_k1/pct50_k1
        rat7550_2 = pct75_k2/pct50_k2
        
        
        open(4,file='targets.out',form='formatted')
        write(4,832) ratbigh
        write(4,832) corrwh
        write(4,832) rathouse
        write(4,832) rat7550_1
        write(4,832) rat7550_2
832     format(f18.12)
        close(4)
        
        open(4,file='percentiles.out',form='formatted')
        write(4,109) pct10_k1,pct25_k1,pct50_k1,pct75_k1,pct90_k1
        write(4,109) pct10_k2,pct25_k2,pct50_k2,pct75_k2,pct90_k2
        write(4,108) pct25s_k2,pct75s_k2
109     format(5f12.6)
108     format(2f12.6)
        close(4)

      return
      end

	   subroutine funca(np,j,x,fj)
      use parameters_mod
      use states_mod
      use targets_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: fj
      real*8 :: xhp,c,it,phi,utilc,yval,yp,ydp,ab,humstay,getxhp,p
      integer :: khi,klo,ierr
      logical :: skip
      
      skip = .true.
      
      it = x(1)
      c = x(2)
      phi = (theta/pstay)**(one/nus)*(c**(nuc/nus))
      
      ab = apts(naps)

      xhp = getxhp(it,hcap(nk),ab,nk)
     
      if ((nuc .eq. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = dlog(c)+theta*dlog(phi)
         
      elseif ((nuc .ne. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = (c**(one-nuc))/(one-nuc)+theta*dlog(phi)
         
      elseif ((nuc .eq. 1.0D+00) .and. (nus .ne. 1.0D+00)) then
         utilc = dlog(c)+theta*(phi**(one-nus))/(one-nus)
         
      else
         utilc = (c**(one-nuc))/(one-nuc)+theta*(phi**(one-nus))/(one-nus) 
           
      endif
      
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,1,0,0,naps,nk)

      fj = -(utilc+beta*yval)

      return
      end


      subroutine dfunc(np,j,x,fj,dummy)
      use parameters_mod
      use states_mod
      use targets_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: fj
      real*8 :: xhp,c,it,yval,phi,yp,ydp,ab,humstay,getxhp,dhpdi,getdxhp,dphidc
      integer :: khi,klo
      external dummy
      
      
      it = x(1)
      c = x(2)
      phi =(theta/pstay)**(one/nus)*(c**(nuc/nus))
      
      ab = apts(naps)
      xhp = getxhp(it,hcap(nk),ab,nk)
      dhpdi = getdxhp(it,hcap(nk),ab,nk)
      dphidc = (nuc/nus)*((theta/pstay)**(one/nus))*c**((nuc/nus)-one)
      
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,0,1,0,naps,nk)

      fj(1) = -beta*yp*dhpdi
      fj(2) = -c**(-nuc)-theta*dphidc*phi**(-nus)
      
      return
      end

    
      subroutine const(np,j,x,gj)
      use parameters_mod
      use states_mod
      use targets_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: gj
      real*8 :: xhp,c,it,n,phi
      integer :: khi,klo
      
      it = x(1)
      c = x(2)
      phi =(theta/pstay)**(one/nus)*(c**(nuc/nus))

      gj = c+it+pstay*phi-wage*xhcur
        
      return
      end
      
                
      subroutine dconst(np,j,x,gj,dummy)
      use parameters_mod
      use states_mod
      use targets_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: gj
      real*8 ::c,phi,dphidc
      integer :: khi,klo
      external dummy
      
      c = x(2)
      phi =(theta/pstay)**(one/nus)*(c**(nuc/nus))
      dphidc = (nuc/nus)*((theta/pstay)**(one/nus))*c**((nuc/nus)-one)
      
      gj(1) = one
      gj(2) = one+pstay*dphidc
      
      return
      end
      
      
      double precision function getxhp(inv,hum,ab,nbd)
      use parameters_mod
      use targets_mod
      implicit none
      real*8 :: inv,hum,ab,elas,sh_a,sh_i,sh_H,biga,gam
      integer :: nbd
         
      gam = gam1 
      
      if (nbd .eq. 1) then
        biga = biga1
      else
        biga = biga2
      endif
      
      sh_a = lam 
      sh_i = lam 
      sh_H = lam
      
      if (inv .lt. 1.0D-08) then
        getxhp = 1.0D-08
      else  
          getxhp = biga*(sh_a*ab**gam+sh_i*inv**gam+sh_H*hum**gam)**(one/gam)
      endif
        
      return
      end
      
      double precision function getdxhp(inv,hum,ab,nk)
      use parameters_mod
      use targets_mod
      implicit none
      real*8 :: inv,hum,ab,elas,bigf,sh_a,sh_i,sh_H,biga,gam
      integer :: nk
      
      gam = gam1  
      
      if (nk .eq. 1) then
        biga = biga1  
      else
        biga = biga2
      endif
    
      sh_a = lam
      sh_i = lam
      sh_H = lam
      
      bigf = biga*(sh_a*ab**(gam)+sh_i*inv**gam+sh_H*hum**gam)**((one/gam)-one) 
      getdxhp = bigf*one*sh_i*inv**(gam-one)
      
      return
      end
      
      double precision function zbrent(func,x1,x2,tol)
      implicit none
      real*8 :: x1,x2,tol,zip,fa,fb,a,b,c,fc,d,e,tol1,xm,s,p,q,r,zz
      integer :: iter
      real*8, parameter :: eps = 3.0D-08
      integer, parameter :: itmax = 300 
      interface
         double precision function func(x)
         implicit none
         real*8, intent(in) :: x
         end function func
      end interface

      a = x1
      b = x2
      fa = func(a)
      fb = func(b)
      zip = 0.0D+00

      if ((fa > zip .and. fb > zip) .or. (fa < zip .and. fb < zip)) then
      print*,'root must be bracketed'
      print*, a,b
      print*, fa,fb
      endif

      c = b
      fc = fb
      do iter = 1,itmax
        if (iter .eq. itmax) print*, 'HIT ITMAX'
         if ((fb > zip .and. fc > zip) .or. (fb < zip .and. fc < zip)) then
            c = a
            fc = fa
            d = b-a
            e = d
         endif
         if (dabs(fc) < dabs(fb)) then
            a = b
            b = c
            c = a
            fa = fb
            fb = fc
            fc = fa
         endif
         tol1 = 2.0D+00*eps*dabs(b)+0.5D+00*tol
         xm = 0.5D+00*(c-b)
         if (dabs(xm) <= tol1 .or. fb == zip) then
            zbrent = b
            return
         endif
         if (dabs(e) >= tol1 .and. dabs(fa) > dabs(fb)) then
            s = fb/fa
            if (a == c) then
               p = 2.0D+00*xm*s
               q = 1.0D+00-s
            else
               q = fa/fc
               r = fb/fc
               p = s*(2.0D+00*xm*q*(q-r)-(b-a)*(r-1.0D+00))
               q = (q-1.0D+00)*(r-1.0D+00)*(s-1.0D+00)
            endif
            if (p > zip) q = -q
            p = dabs(p)
            if (2.0D+00*p < dmin1(3.0D+00*xm*q-dabs(tol1*q),dabs(e*q))) then
               e = d
               d = p/q
            else
               d = xm
               e = d
            endif
         else
            d = xm
            e = d
         endif
         a = b
         fa = fb
         if (dabs(d) > tol1) then
            b = b+d
         else
            b = b+sign(tol1,xm)
         endif
         fb = func(b)
      end do

      pause 'zbrent exceeding maximum iterations'
      zbrent = b

      return
      end
      
    subroutine sort2(n,arr,brr)
	implicit none
	integer, intent(in) :: n
	real*8, dimension(n) :: arr,brr
	integer, parameter :: m = 7
	integer, parameter :: nstack = 50
	integer, dimension(nstack) :: istack
    integer :: jstack,l,ir,i,j,k
    real*8 :: a,b,temp

	jstack = 0
	l = 1
	ir = n
 1    if (ir-l < m) then
         do j = l+1,ir
	      a = arr(j)
	      b = brr(j)
	      do i = j-1,l,-1
	         if (arr(i) <= a) goto 2
	         arr(i+1) = arr(i)
	         brr(i+1) = brr(i)
          end do
            i = l-1
 2          arr(i+1) = a
            brr(i+1) = b
         end do
         if (jstack == 0) return
	   ir = istack(jstack)
	   l = istack(jstack-1)
	   jstack = jstack-2
	else
	   k = (l+ir)/2
	   temp = arr(k)
	   arr(k) = arr(l+1)
	   arr(l+1) = temp
	   temp = brr(k)
	   brr(k) = brr(l+1)
	   brr(l+1) = temp
	   if (arr(l) > arr(ir)) then
	      temp = arr(l)
	      arr(l) = arr(ir)
	      arr(ir) = temp
	      temp = brr(l)
	      brr(l) = brr(ir)
	      brr(ir) = temp
	   endif
	   if (arr(l+1) > arr(ir)) then
	      temp = arr(l+1)
	      arr(l+1) = arr(ir)
	      arr(ir) = temp
	      temp = brr(l+1)
	      brr(l+1) = brr(ir)
	      brr(ir) = temp
	   endif
	   if (arr(l) > arr(l+1)) then
	      temp = arr(l)
	      arr(l) = arr(l+1)
	      arr(l+1) = temp
	      temp = brr(l)
	      brr(l) = brr(l+1)
	      brr(l+1) = temp
	   endif
	   i = l+1
	   j = ir
	   a = arr(l+1)
	   b = brr(l+1)
 3       continue
            i = i+1
	   if (arr(i) < a) goto 3
 4       continue
            j = j-1
	   if (arr(j) > a) goto 4
	   if (j < i) goto 5
	   temp = arr(i)
	   arr(i) = arr(j)
	   arr(j) = temp
	   temp = brr(i)
	   brr(i) = brr(j)
	   brr(j) = temp
	   goto 3
 5       arr(l+1) = arr(j)
         arr(j) = a
	   brr(l+1) = brr(j)
         brr(j) = b
	   jstack = jstack+2
	   if (jstack > nstack) pause 'nstack too small in sort2'
	   if (ir-i+1 >= j-l) then
	      istack(jstack) = ir
	      istack(jstack-1) = i
	      ir = j-1
	   else
	      istack(jstack) = j-1
	      istack(jstack-1) = l
	      l = i
	   endif
	endif
	goto 1

	end
   
     double precision function upperb(x)
	  use parameters_mod
	  use states_mod
	  use targets_mod
	  implicit none
	  real*8, intent(in) :: x
	  real*8 :: phi
	  
	  phi = (theta/pstay)**(one/nus)*(x**(nuc/nus))
	  upperb = pstay*phi+x-wcap*xhcur
 
	  return
	  end
	  
	  subroutine simpath
	  use parameters_mod
	  use states_mod
	  implicit none	  
	  real*8, dimension(napts,napts) :: pi2
      real*8 :: shock
	  integer :: i,j,count,a0,at,jlo
	  integer, dimension(nsim) :: p
	    
      call random_seed(put=nseed)
      
	  a0 = 5  ! Change when napts changes
	  at = a0
	  
	  do i = 1,napts
	    pi2(i,1) = pi(i,1)
	    do j = 2,napts
	      pi2(i,j) = pi2(i,j-1) + pi(i,j)
	    end do
	    if (dabs(one-pi2(i,napts)) .gt. 1.0D-06) then 
	       print*, 'cumulative pi2 matrix not summing to one',i
	       print*, pi2(i,:)
	       pause
	    endif 
	  enddo
	  
      jlo = 100
      
	  do i = 1,nsim
      
        call RANDOM_NUMBER(shock)
        
        call hunt(pi2(at,:),napts,shock,jlo)
        
        if (jlo .lt. 1) then
                 at = 1
             elseif (jlo .ge. napts) then
                 at = napts
             else
                 at = jlo+1
             endif

        p(i) = at  
	  end do
	   
	   apath = p
	   
	   open(3,file='apath.out',form='formatted')
	   do i = 1,nsim
	      write(3,291) i,apath(i) 
291       format(i7,i4)
       end do
       close(3)	      
	   
	  return
	  end
	  
	  subroutine simh
	  use parameters_mod
	  use states_mod
	  use targets_mod
	  implicit none
	  real*8 :: hp,rsqa,sderra
	  real*8, dimension(2) :: a0new
	  real*8, dimension(nsim+1) :: hpath
	  real*8, dimension(nsim-ndrop-1,2) :: xdata
	  real*8, dimension(nsim-ndrop-1,1) :: ydata
	  integer :: i,jlo,npts,ap
	  
	  hp = hcap(2)
	  hpath(1) = hp
	  
	  do i = 2,nsim+1
	    ap = apath(i-1)
	    call hunt(h2pts,nh2pts,hp,jlo)
	    hp = opthg(jlo,ap,2)
	    hpath(i) = hp
	  end do
 
	    npts = 1
	  do i = ndrop+1,nsim
	     xdata(npts,1) = one
	     xdata(npts,2) = dlog(wfix*hpath(i))
         ydata(npts,1) = dlog(wfix*hpath(i+1))
         npts = npts+1
      end do
      
      open(3,file='regression.out',form='formatted')
      do i = ndrop+1,nsim
         write(3,391) i,ydata(i,1),xdata(i,:)
391      format(i7,3f18.10)
      end do
      close(3)   
      
      call ols2(ydata,xdata,nsim-ndrop-1,a0new,rsqa,sderra,npts-1)
	  
	  corrwh = a0new(2)
	  print*, 'reg coeff'
	  print*, a0new

	  return 
	  end
	    