!!!!  Computes transition path from initial steady state to final steady state when households make decisions myopically

!**********************************************************************************************************************************
!PARAMETERS
!**********************************************************************************************************************************
 
      module parameters_mod
      implicit none
     
      integer, parameter :: ncapt = 20  
      integer, parameter :: nhpts = 150 
      integer, parameter :: hadd = 9
      integer, parameter :: nh2pts = 10000
      integer, parameter :: nkpts = 2
      integer, parameter :: napts = 9
      integer, parameter :: nlpts = 300
      integer, parameter :: nread4 = 0  ! reads in transition guess
      integer, parameter :: mxiter = 300
      integer, parameter :: mxiter2 = 500
      integer, parameter :: mxpol = 200
      integer, parameter :: nlzpts = 500
      integer, parameter :: iwsize = 50
      integer, parameter :: nwsize = 500
      integer, parameter :: nparam = 2
      integer, parameter :: neq = 0
      integer, parameter :: nf = 1
      integer, parameter :: nineq = 1
      integer, parameter :: nineqn = 1
      integer, parameter :: neqn = 0
      integer, parameter :: ncal = 4
      integer, parameter :: bigaswitch = 1  ! 0 - Wilson Case, 1 - MLK Case
      real*8, parameter :: costm = 0.0D+00  ! Set to a very large number for X case
      
      real*8, parameter :: one = 1.0D+00
      real*8, parameter :: biga2 = 5.00D+00 
      real*8, parameter :: mu = 0.64D+00
      real*8, parameter :: beta = 0.665D+00
      real*8, parameter :: wfix = 1.0D+00
      real*8, parameter :: nuc = 1.5D+00
      real*8, parameter :: nus = 2.0D+00 
      real*8, parameter :: pmin =  1.0D-06
      real*8, parameter :: phom = 0.05D+00
      real*8, parameter :: phom2 = 0.01D+00
      real*8, parameter :: stig1 = 1.0D-03
      real*8, parameter :: stig2 = 1.0D-03
      real*8, parameter :: bighmin = 1.0D-02
      real*8, parameter :: lfix1 = 0.114D+00
      real*8, parameter :: lfix2 = one-lfix1
      real*8, parameter :: ptol = 2.0D-02
      
      real*8, parameter :: lam = one/3.0D+00
      
      
      real*8, dimension(napts,napts) :: pi
      real*8, dimension(nlpts) :: lpts
      
      end module parameters_mod
      

!**********************************************************************************************************************************
!DECISION RULES, etc
!**********************************************************************************************************************************      
      module states_mod
        use parameters_mod
        implicit none
        integer :: nbdstay,nbdgo,nap,nk,nh,nt
        real*8 :: xhcur,pstay,pgo,pop1,pop2,trans,xn,wage,bighstay,bighgo
        real*8 :: siga,rho,theta,gam1,biga1
        real*8 :: cont,phit,invt,ht,vtrans
        real*8, dimension(nhpts,napts,nkpts) :: y2,opth2,optk2,optc2,optphi2,opti2
        real*8, dimension(nhpts,napts,nkpts,ncapt) :: y2g,opthg,optkg,optcg,optphig,optig
        real*8, dimension(nhpts) :: hpts
        real*8, dimension(nhpts,napts,nkpts) :: vhdp
        real*8, dimension(nhpts) :: upbnd1,upbnd2
        real*8, dimension(nh2pts) :: h2pts
        real*8, dimension(napts) :: apts
        real*8, dimension(nkpts) :: kpts
        real*8, dimension(nkpts) :: pguess,phat,hbelief,hhat
        real*8, dimension(nh2pts,napts,nkpts) :: fstart,fthat,ft,fnrm
        real*8, dimension(nhpts,napts,nkpts) :: v,vdp
        real*8, dimension(nkpts,ncapt) :: hcap,pcap
        real*8, dimension(nkpts,ncapt) :: bighum,bigc,bigq,meana,pops,hp,bigi
        
      end module states_mod
      
      
!**********************************************************************************************************************************
!MAIN PROGRAM
!**********************************************************************************************************************************    
        program main
        use parameters_mod
        use states_mod
        implicit none  
        integer :: i,iii,j,jj,k,kk,kkk,multpl,dumi,khi,klo,pflag
        real*8 :: hinc,littlehouse,zbrent,dum,eusum,pdiff01,pdiff11,pdiff02,pdiff12,hb1,hb2
        real*8, dimension(nh2pts,napts,nkpts) :: f1
        real*8, dimension(ncal) :: xcal
        real*8 :: getxhp
     
!***********************************************************     
!Build human capital grids
!***********************************************************
   
      open(2,file='hgrid.in',form='formatted')
        do i = 1,nhpts
            read(2,458) hpts(i)
458         format(f18.10)
        end do
      close(2)
      
      open(3,file='nbd_calib.best',form='formatted')
      do i = 1,ncal
         read(3,132) xcal(i),dum
 132     format(2f18.10)
      end do
      close(3)
      
      biga1  = xcal(1)
      siga   = xcal(2)
      theta = xcal(3)
      gam1  = xcal(4)
      
      h2pts(1) = hpts(1)
      hinc = (hpts(nhpts)-hpts(1))/dble(real(nh2pts-1))
      do i = 2,nh2pts
         h2pts(i) = h2pts(i-1)+hinc
      end do
      
      kpts(1) = one
      kpts(2) = 2.0D+00
      
      rho = 0.0D+00
      
      apts(1) = -sqrt(dble(napts-1))*siga/sqrt(one-rho*rho)
      apts(napts) = -apts(1)
      do i = 2,napts-1
         apts(i) = apts(i-1)+(apts(napts)-apts(1))/dble(napts-1)
      end do
      apts = dexp(apts)
      
      call rouwenhorst(rho,pi,napts)


!***********************************************************     
! Distribution of human capital
!***********************************************************
         open(2,file='saving.fst',form='formatted')
         do k = 1,nkpts
            do i = 1,nh2pts
               read(2,547) dum,dum,f1(i,:,k)
 547           format(11f18.10)            
            end do
         end do
         close(2)
         
         fstart = f1

!***********************************************************     
! Joint human capital stocks
!***********************************************************
     open(2,file='saving_init.agg',form='formatted')
        do k = 1,nkpts
          read(2,391) dum,dum,hcap(k,1),dum,pcap(k,1),dum,dum,dum,dum 
391       format(i4,9f16.10)
        end do
     close(2)
     
     
     hb1 = 0.0D+00
     hb2 = 0.0D+00
     pop1 = 0.0D+00
     pop2 = 0.0D+00
     
     do i = 1,nh2pts
        do j = 1,napts 
           do k =1,nkpts
              pop1 = pop1 + fstart(i,j,1)
              pop2 = pop2 + fstart(i,j,2)
              hb1 = hb1 + h2pts(i)*fstart(i,j,1)
              hb2 = hb2 + h2pts(i)*fstart(i,j,2)
           end do
         end do
     end do
     
     hbelief(1) = hb1/pop1
     hbelief(2) = hb2/pop2
     
     pguess(1) = 1.182807554D+00
     pguess(2) = 2.054037669D+00
   
      open(16,file='myopic_transition.out',form='formatted')
      do kkk = 1,ncapt
         nt = kkk
         pflag = 0
         pdiff01 = 100.0D+00
         pdiff02 = 100.0D+00
         do iii = 1,100
            ! Solve household problem in period kkk and get prices
            call zeta2
            pdiff11 = dabs(pguess(1)-phat(1))
            pdiff12 = dabs(pguess(2)-phat(2))
            
            if (dabs(pguess(1)-phat(1)) .lt. ptol .and. dabs(pguess(2)-phat(2)) .lt. ptol  ) goto 3
            
            if (pdiff11-pdiff01 .gt. 0.0D+00 .or. pdiff12-pdiff02 .gt. 0.0D+00 .and. iii .gt. 5) pflag = 1
            
            if (pflag .eq. 0) then
               pguess = (one-phom)*pguess+phom*phat  
            else
                pguess = (one-phom2)*pguess+phom2*phat
            endif
                 
            pdiff01 = pdiff11
            pdiff02 = pdiff12
            
            if (multpl(iii,1) .eq. 1) then
	            write(6,322) kkk,iii,dabs(pguess(1)-phat(1)),dabs(pguess(2)-phat(2)),pop1,pop2,pflag
 322           format(2i5,4f12.8,i3)
            endif
            
         end do
  
3           pcap(:,kkk) = pguess
            call calcf
    
            fstart = ft
            hcap(:,kkk) = hhat
            
            write(6,574) kkk,pguess,hbelief,hhat
574         format(i5,6f12.8)
            
            write(16,743) kkk,pcap(:,kkk),hbelief,hcap(:,kkk),bigq(:,kkk),bigc(:,kkk),pop1,pop2
743         format(i4,12f18.12)
            
            hb1 = 0.0D+00
            hb2 = 0.0D+00
            pop1 = 0.0D+00
            pop2 = 0.0D+00
            
            do i = 1,nh2pts
               do j = 1,napts 
                  do k =1,nkpts
                    pop1 = pop1 + fstart(i,j,1)
                    pop2 = pop2 + fstart(i,j,2)
                    hb1 = hb1 + h2pts(i)*fstart(i,j,1)
                    hb2 = hb2 + h2pts(i)*fstart(i,j,2)
                 end do
               end do
            end do
           
            hbelief(1) = hb1/pop1
            hbelief(2) = hb2/pop2
       
      end do
      
      close(16)
1     print*, '----------------------------------------------'
  

      open(3,file='trans_path.out',form = 'formatted')
      do i = 1,ncapt
         write(3,939) i, pcap(1,i), pcap(2,i), hcap(1,i), hcap(2,i),bigq(1,i),bigq(2,i),bigc(1,i),bigc(2,i),pops(1,i),pops(2,i)
 939     format(i4,10f18.10)
      end do
      close(3)
      
      do i = 1,ncapt
         write(6,938) i, pcap(1,i), pcap(2,i), hcap(1,i), hcap(2,i) 
938     format(i5,4f18.10)         
      end do
      
      open(2,file='vopt.out',form='formatted')
        do kk = 1,ncapt
          do k = 1,nkpts
             do j = 1,napts
                do i = 1,nhpts
                    write(2,902) kk, hpts(i), apts(j), kpts(k),optkg(i,j,k,kk),y2g(i,j,k,kk),opthg(i,j,k,kk),optig(i,j,k,kk),optcg(i,j,k,kk),optphig(i,j,k,kk)
902                 format(i4,2f18.10,2f5.1,5f18.10)
                end do
             end do
          end do
        end do
        close(2)
      
      
      print*, 'all done'
      
      pause
 
      end
  
  
  
!**********************************************************************************************************************************
!Subroutines and Functions
!**********************************************************************************************************************************  
     
     subroutine zeta2
     use parameters_mod
     use states_mod
	  implicit none
	  real*8, dimension(nhpts,napts,nkpts) :: y,opth,optc,optphi,opti,optk
     real*8, dimension(nh2pts,napts,nkpts) :: optcfine,optphifine,optifine,optkfine
	  integer :: i,j,k,kkk,iii,jlo1,jlo2,multpl,mode,iprint,miter,inform,jk,kk,ierr
	  real*8 :: wgt1,wgt2,fsum,eps,epsneq,udelta,bigbnd,ck,gk,p1,p2,kstay,kgo,neps &
	            ,cstay,cgo,hstay,hgo,istay,igo,ystay,ygo,w,phistay,phigo,bigh1,bigh2,bigq1,bigq2 &
	            ,bigc1,bigc2,bigqcheck,littlehouse,getxhp,upcstay,upcgo,zbrent2,stig,hp1,hp2,bigi1,bigi2,test,differ,vmxdif,amxdif,bmxdif,dmxdif
      real*8, dimension(nwsize) :: work
      real*8, dimension(nparam) :: x,bl,bu,bl2,bu2,bcheck
      real*8, dimension(nf+1) :: ff
      real*8, dimension(nineq+neq+1) :: g
      integer, dimension(nwsize) :: iw
      external func,func2,const,const2,dconst,dconst2,grobfd,grcnfd,dfunc,dfunc2
      interface
      double precision function upperb(x)
        real*8, intent(in) :: x
      end function upperb
      end interface
     
      mode = 100
      iprint = 0
      miter = 500
      eps = 1.0D-12
      epsneq = 1.0D-12
      udelta = 0.0D+00
	   bigbnd= 1.0D+10
     
 
      wage = wfix
      littlehouse = 1.0D-08
      
         if (nuc .eq. 1.0D+00) then
            do i = 1,nhpts
               xhcur = hpts(i)
               upbnd1(i) = 0.9999D+00*wfix*xhcur
               upbnd2(i) = upbnd1(i)
            end do
         else  
            do i = 1,nhpts
               xhcur = hpts(i)
               upbnd1(i) = zbrent2(upperb,1.0D-08,wfix*xhcur,pguess(1),1.0D-08)
               upbnd2(i) = zbrent2(upperb,1.0D-08,wfix*xhcur,pguess(2),1.0D-08)
            end do
         endif 
         
      do iii = 1,mxiter
         y = y2
         opti = opti2
         optc = optc2
         optk = optk2
         optphi = optphi2

          v = 0.0D+00
              do i = 1,napts
                 do j = 1,napts
                   do k = 1,nkpts
                    v(:,i,k) = v(:,i,k)+pi(i,j)*y(:,j,k)
                   end do
                end do
              end do

        call interp(hpts,v,vdp)
        
        do j = 1,napts
            nap = j
            do k = 1,nkpts
               nk = k
                if (k .eq. 1) then
                    kstay = 1.0D+00         
                    kgo = 2.0D+00            
                    bighstay = hbelief(1)     
                    bighgo = hbelief(2)
                    pstay = pguess(1)
                    pgo = pguess(2)
                    nbdstay = 1
                    nbdgo = 2
                elseif (k .eq. 2) then
                    kstay = 2.0D+00
                    kgo = 1.0D+00
                    bighstay = hbelief(2)     
                    bighgo = hbelief(1)
                    pstay = pguess(2)
                    pgo = pguess(1)
                    nbdstay = 2
                    nbdgo = 1
                endif              
                 
	        do i = 1,nhpts
               
               if (k .eq. 1) then 
                   upcstay = upbnd1(i)
                   upcgo = upbnd2(i)
               else
                   upcstay = upbnd2(i)
                   upcgo = upbnd1(i)
               endif  
                 
 	           xhcur = hpts(i)
               bl(1) = 1.0D-08
               bl(2) = 1.0D-08
                
               bu(1) = wage*xhcur*0.9999D+00-pstay*littlehouse
               bu(2) = upcstay
 
               bcheck = bu-bl

                if (minval(bcheck) .lt. 1.0D-08) then
                   print*, 'error on constraint set for staying'
                   print*, bl
                   print*, bu
                   print*, wage,xhcur,pstay
                   print*, upbnd1(i),upbnd2(i)
                   pause
                endif

               if (i .eq. 1) then
                  x(1) = 0.0861066999D+00
                  x(2) = 0.0630983908D+00
               else
                  x(1) = opti2(i,j,k)
                  x(2) = optc2(i,j,k)
               endif

               call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, & 
                          epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,func,const,dfunc,dconst)
   
               istay = x(1)
               cstay = x(2)
               hstay = getxhp(istay,bighstay,apts(nap),nbdstay)
               
               phistay = (theta/pstay)**(one/nus)*(x(2)**(nuc/nus))
               ystay = -ff(1)
               
               
                if (i .eq. 1) then
                  x(1) = 0.0146066999D+00
                  x(2) = 0.0717863908D+00
               else
                  x(1) = opti2(i,j,k)
                  x(2) = optc2(i,j,k)
               endif
               
                bl2(1) = 1.0D-08
                bl2(2) = 1.0D-08
                
                bu2(1) = wage*xhcur*0.9999D+00-pgo*littlehouse-costm
                bu2(2) = upcgo-costm
                
                bcheck = bu2-bl2
                
             if (minval(bcheck) .lt. 1.0D-08) then
                   cgo = 1.0D-08
                   igo = 1.0D-08
                   phigo = (theta/pgo)**(one/nus)*(cgo**(nuc/nus))
                   ygo = -1.0D+06
                   hgo = hpts(1)
              else
               call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
                          epsneq,udelta,bl2,bu2,x,ff,g,iw,iwsize,work,nwsize,func2,const2,dfunc2,dconst2)
!                          
!               call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
!                          epsneq,udelta,bl2,bu2,x,ff,g,iw,iwsize,work,nwsize,func2,const2,grobfd,dconst2)
               
               igo = x(1)
               cgo = x(2)
               hgo = getxhp(igo,bighgo,apts(nap),nbdgo)
               phigo = (theta/pgo)**(one/nus)*(x(2)**(nuc/nus))
               ygo = -ff(1)          
            endif        
             

               if (ystay .gt. ygo - stig) then 
                  opti2(i,j,k) = istay
                  opth2(i,j,k) = hstay
                  optc2(i,j,k) = cstay
                  optk2(i,j,k) = kstay
                  phistay = (theta/pstay)**(one/nus)*(cstay**(nuc/nus))
                  optphi2(i,j,k) = phistay
                  y2(i,j,k) = ystay  
               endif
                 if (ygo .gt. ystay + stig) then
                  opti2(i,j,k) = igo
                  opth2(i,j,k) = hgo
                  optc2(i,j,k) = cgo
                  optk2(i,j,k) = kgo
                  phigo = (theta/pgo)**(one/nus)*(cgo**(nuc/nus))
                  optphi2(i,j,k) = phigo
                  y2(i,j,k) = ygo
                endif
    
                end do
            end do
         end do
         
         differ = 0.0D+00
         
 2       vmxdif = maxval(dabs(y2-y))
         amxdif = maxval(dabs(opti2-opti))
         bmxdif = maxval(dabs(optc2-optc))
         dmxdif = maxval(dabs(optk2-optk))
         
         if (vmxdif .lt. 1.0D-05 .and. iii .gt. 2) goto 8
         
         if (multpl(iii,10) .eq. 1) then
            write(6,888) iii,mxiter,vmxdif,amxdif,bmxdif,dmxdif
 888        format(2i5,4f12.7)
         endif
      end do
      
8       y2g(:,:,:,nt) = y2
        optig(:,:,:,nt) = opti2
        opthg(:,:,:,nt) = opth2
        optcg(:,:,:,nt) = optc2
        optkg(:,:,:,nt) = optk2
        optphig(:,:,:,nt) = optphi2
        
        ft = fstart
	     fthat = 0.0D+00
  
        ! GET THE INTRATEMPORAL DISTRIBUTION 
	     do i = 1,nh2pts
	        do j = 1,napts
	           do k = 1,nkpts
	             call hunt(hpts,nhpts,h2pts(i),jlo1)
	             if (jlo1 .lt. 1) jlo1 = 1
                 if (jlo1 .gt. nhpts) jlo1 = nhpts-1    
                 ck = (h2pts(i)-hpts(jlo1))/(hpts(jlo1+1)-hpts(jlo1))
                 optphifine(i,j,k) = (one-ck)*optphi2(jlo1,j,k)+ck*optphi2(jlo1+1,j,k)
                 optcfine(i,j,k) = (one-ck)*optc2(jlo1,j,k)+ck*optc2(jlo1+1,j,k)
                 optifine(i,j,k) = (one-ck)*opti2(jlo1,j,k)+ck*opti2(jlo1+1,j,k)
	              optkfine(i,j,k) = (one-ck)*optk2(jlo1,j,k)+ck*optk2(jlo1+1,j,k)
	              if (optkfine(i,j,k) .lt. 1.0D+00) optkfine(i,j,k) = 1.0D+00
                 if (optkfine(i,j,k) .gt. 2.0D+00) optkfine(i,j,k) = 2.0D+00
                 wgt2 = 2.0D+00 - optkfine(i,j,k)
                 fthat(i,j,1) = fthat(i,j,1) + wgt2*ft(i,j,k)  
                 fthat(i,j,2) = fthat(i,j,2) + (one-wgt2)*ft(i,j,k)
              end do
            end do
          end do

          ! Check prices 
          
        pop1 = 0.0D+00
        pop2 = 0.0D+00
  
        do i = 1,nh2pts
            do j = 1,napts
            ! Correct tiny numerical errors
                if (fthat(i,j,1) .lt. 1.0D-08 .and. fthat(i,j,1) .gt. 0.0D+00) then
                   fthat(i,j,2) = fthat(i,j,2) + fthat(i,j,1)
                   fthat(i,j,1) = 0.0D+00
                endif
                pop1 = pop1 + fthat(i,j,1)
                pop2 = pop2 + fthat(i,j,2)
            end do
        end do
         
        if (pop1 .lt. 1.0D-08) pop1 = 1.0D-08
        if (pop2 .lt. 1.0D-08) pop2 = 1.0D-08
      
        pops(1,nt) = pop1
        pops(2,nt) = pop2
        
        
	     fsum = sum(fthat)
	     if (dabs(fsum-one) .gt. 1.0D-06) then
	        write(6,740) fsum
 740        format('integration failure in fthat: ',f18.10)
            pause
	     endif
	     
        bigh1 = 0.0D+00
        bigh2 = 0.0D+00
        bigq1 = 0.0D+00
        bigq2 = 0.0D+00
        bigc1 = 0.0D+00
        bigc2 = 0.0D+00
        bigqcheck = 0.0D+00

        fnrm(:,:,1) = fthat(:,:,1)/pop1   ! Normalized intratemporal distributions
        fnrm(:,:,2) = fthat(:,:,2)/pop2

        do j = 1,napts
            do i = 1,nh2pts
                bigh1 = bigh1+h2pts(i)*fnrm(i,j,1)
                bigh2 = bigh2+h2pts(i)*fnrm(i,j,2) 
                bigc1 = bigc1 + optcfine(i,j,1)*fnrm(i,j,1)
                bigc2 = bigc2 + optcfine(i,j,2)*fnrm(i,j,2)
                bigq1 = bigq1 + optphifine(i,j,1)*fthat(i,j,1)  
                bigq2 = bigq2 + optphifine(i,j,2)*fthat(i,j,2)  
                bigi1 = bigi1 + optifine(i,j,1)*fnrm(i,j,1)
                bigi2 = bigi2 + optifine(i,j,2)*fnrm(i,j,2)                           
            end do
        end do

        if (bigq1 .lt. 1.0d-08) then
           bigq1 = 1.0d-08
        endif
        if (bigq2 .lt. 1.0d-08) then 
           bigq2 = 1.0d-08
        endif
        
        hhat(1) = bigh1
        hhat(2) = bigh2
        bigq(1,nt) = bigq1
        bigq(2,nt) = bigq2
        bigc(1,nt) = bigc1
        bigc(2,nt) = bigc2
  
        phat(1) = (wfix/mu)*(bigq1/lfix1)**((one-mu)/mu)
        phat(2) = (wfix/mu)*(bigq2/lfix2)**((one-mu)/mu) 
        
        if (phat(1) .lt. pmin) phat(1) = pmin
        if (phat(2) .lt. pmin) phat(2) = pmin
        if (pop1 .le. 1.0D-08) phat(1) = pmin
        if (pop2 .le. 1.0D-08) phat(2) = pmin
        
        if (phat(1) .gt. 10.0D+00) phat(1) = 10.0D+00
        if (phat(2) .gt. 10.0D+00) phat(2) = 10.0D+00
    
      return 
      end
      
   
      subroutine calcf     
      use parameters_mod
      use states_mod
      implicit none
      integer :: i,j,k,kk,jlo1,jlo2,nbd
      real*8 :: hp1,hp2,ck,gk,wgt1,bigh1,bigh2,fsum
      real*8, dimension(nh2pts,napts,nkpts) :: opthfine
      real*8 :: getxhp
  
           ! CONSTRUCT BEGINNING OF NEXT PERIOD DISTRIBUTION     
        ft = 0.0D+00    
        
        ! Construct opth2 based on optk2, opti2, and the actual (H1,H2) not hbelief         
        jlo1 = 1
        jlo2 = 1
        
        do i = 1,nhpts 
           do j = 1,napts 
              do k = 1,nkpts
                 nbd = int(optk2(i,j,k))
                 opth2(i,j,k) = getxhp(opti2(i,j,k),hhat(nbd),apts(j),nbd) 
              end do
           end do
        end do
        
        ! Build next period distribution
	     do i = 1,nh2pts
	        do j = 1,napts
	           do k = 1,nkpts
                call hunt(hpts,nhpts,h2pts(i),jlo1) 
                if (jlo1 .lt. 1) jlo1 = 1
                if (jlo1 .gt. nhpts) jlo1 = nhpts-1    
                    ck = (h2pts(i)-hpts(jlo1))/(hpts(jlo1+1)-hpts(jlo1))
                    gk = (one-ck)*opth2(jlo1,j,k)+ck*opth2(jlo1+1,j,k)
                    opthfine(i,j,k) = gk
                    
                    call hunt(h2pts,nh2pts,gk,jlo2)
                
                    if (jlo2 .lt. 1) then
                       jlo2 = 1
                       wgt1 = 1.0D+00
                    elseif (jlo2 .ge. nh2pts) then
                       jlo2 = nh2pts-1
                       wgt1 = 0.0D+00
                    else   
                       wgt1 = one-(gk-h2pts(jlo2))/(h2pts(jlo2+1)-h2pts(jlo2))
                    endif
	           
	             if (wgt1 .lt. 0.0D+00) then 
	               print*, 'negative wgt1'
	               print*, gk,h2pts(jlo2),h2pts(jlo2+1)-h2pts(jlo2)
	               pause
	             endif
 
                 ! intertemporal distribution (t+1)       
               do kk = 1,napts
               
	              ft(jlo2,kk,k) = pi(j,kk)*wgt1*fthat(i,j,k)+ft(jlo2,kk,k)
	              ft(jlo2+1,kk,k) = pi(j,kk)*(one-wgt1)*fthat(i,j,k)+ft(jlo2+1,kk,k)  

                 end do
	        end do
	     end do
	     end do

        fsum = sum(ft)
        if (dabs(fsum-one) .gt. 1.0D-06) then
	        write(6,739) fsum
 739        format('integration failure in ft: ',f18.10)
           print*, 'fthat sum', sum(fthat)
            pause
	     endif           

  
      return
      end
      
	  subroutine splint2(xa,ya,y2a,x,y,yp,ydp,khi,klo,kvals,n,ndoy,ndoyp,ndoydp,na,nk)
      use parameters_mod
      implicit none
      integer, intent(in) :: n,ndoy,ndoyp,ndoydp,kvals,na
      integer, intent(inout) :: khi,klo
      real*8, intent(in), dimension(n) :: xa
      real*8, intent(in), dimension(n,napts,nkpts) :: ya,y2a
      real*8, intent(in) :: x
      real*8, intent(out) :: y,yp,ydp
      integer :: k,nk
      real*8 :: h,a,b,asq,bsq

      if (kvals == 0) then
         klo = 1
         khi = n
 1       if ((khi-klo) > 1) then
            k = (khi+klo)/2
            if (xa(k) > x) then
               khi = k
            else
               klo = k
            endif
            goto 1
         endif
      endif

      h = xa(khi)-xa(klo)
      a = (xa(khi)-x)/h
      b = (x-xa(klo))/h
      asq = a*a
      bsq = b*b

      if (ndoy == 1) y = a*ya(klo,na,nk)+b*ya(khi,na,nk)+((asq*a-a)*y2a(klo,na,nk)+(bsq*b-b)*y2a(khi,na,nk))*(h*h)/6.0D+00
      if (ndoyp == 1) yp = (ya(khi,na,nk)-ya(klo,na,nk))/h-(3.0D+00*asq-one)/6.0D+00*h*y2a(klo,na,nk)+(3.0D+00*bsq-one)/6.0D+00*h*y2a(khi,na,nk)
      if (ndoydp == 1) ydp = a*y2a(klo,na,nk)+b*y2a(khi,na,nk)
      
      return
      end
      
      subroutine interp(xx,v,vdp)
      use parameters_mod
      implicit none
      real*8, dimension(nhpts), intent(in) :: xx
      real*8, dimension(nhpts) :: a,b,c,r,yy
      real*8, dimension(nhpts,napts,2), intent(in) :: v
      real*8, dimension(nhpts,napts,2), intent(out) :: vdp
      integer :: i,j,k,multpl,nk
     
      a(1) = 0.0D+00
      a(nhpts) = -1.0D+00
      b(1) = 1.0D+00
      b(nhpts) = 1.05D+00
      c(1) = -1.05D+00
      c(nhpts) = 0.0D+00
      r(1) = 0.0D+00
      r(nhpts) = 0.0D+00
      
      do k = 1,nkpts
         nk = k
      do j = 1,napts
         do i = 2,nhpts-1
            a(i) = (xx(i)-xx(i-1))/6.0D+00
            b(i) = (xx(i+1)-xx(i-1))/3.0D+00
            c(i) = (xx(i+1)-xx(i))/6.0D+00
            r(i) = (v(i+1,j,nk)-v(i,j,nk))/(xx(i+1)-xx(i))-(v(i,j,nk)-v(i-1,j,nk))/(xx(i)-xx(i-1))
         end do
         call tridag(a,b,c,r,vdp(:,j,nk),nhpts)
      end do
      end do
    
      return
      end
      
      subroutine tridag(a,b,c,r,u,n)
      implicit none
      real*8, parameter :: toler = 1.0D-12
      integer, parameter :: nmax = 50000
      integer, intent(in) :: n
      real*8, dimension(n) :: a,b,c,r,u
      real*8, dimension(nmax) :: gam
      real*8 :: bet
      integer :: j

      bet = b(1)
      u(1) = r(1)/bet
      do j = 2,n
         gam(j) = c(j-1)/bet
         bet = b(j)-a(j)*gam(j)
         if (dabs(bet) < toler) then
            write(6,100)
 100        format('failure in tridag')
            pause
         endif
         u(j) = (r(j)-a(j)*u(j-1))/bet
      end do
      do j = n-1,1,-1
         u(j) = u(j)-gam(j+1)*u(j+1)
      end do

      return
      end
      
      integer function multpl(i,m)  
      implicit none
      integer, intent(in) :: i,m
      real*8, parameter :: toler = 1.0D-12
      real*8 :: remain

      remain = dble(real(i))/dble(real(m))-i/m

      if (remain .le. toler) then
         multpl = 1
      else
         multpl = 0
      endif

      return
      end
      
      subroutine hunt(xx,n,x,jlo)
      implicit none
      integer, intent(in) :: n
      real*8, intent(in), dimension(n) :: xx
      logical :: ascnd
      real*8,  intent(in) :: x
      integer, intent(inout) :: jlo
      integer :: jhi,inc,jm

      ascnd = xx(n) .ge. xx(1)
      if (jlo .le. 0 .or. jlo .gt. n) then
         jlo = 0
         jhi = n+1
         goto 3
      endif

      inc = 1
      
      if (x .ge. xx(jlo) .eqv. ascnd) then
 1       jhi = jlo+inc
         if (jhi .gt. n) then
            jhi = n+1
         elseif (x .ge. xx(jhi) .eqv. ascnd) then
            jlo = jhi
            inc = inc+inc
            goto 1
         endif
      else
         jhi = jlo
 2       jlo = jhi-inc
         if (jlo .lt. 1) then
            jlo = 0
         elseif (x .lt. xx(jlo) .eqv. ascnd) then
            jhi = jlo
            inc = inc+inc
            goto 2
         endif
      endif
 3    if (jhi-jlo .eq. 1) then
         if (x .eq. xx(n)) jlo = n-1
         if (x .eq. xx(1)) jlo = 1
         return
      endif
      jm = (jhi+jlo)/2
      if (x .ge. xx(jm) .eqv. ascnd) then
         jlo = jm
      else
         jhi = jm
      endif
      goto 3

      end
                                
      double precision function getxhp(inv,hum,ab,nbd)
      use parameters_mod
      use states_mod
      implicit none
      real*8 :: inv,hum,ab,elas,sh_a,sh_i,sh_H,biga,gam
      integer :: nbd
      
      gam = gam1
      
      if (nbd .eq. 1) then
        if (bigaswitch .eq. 1) then
           biga = biga2 
        else
           biga = biga1 
        endif
      else
        biga = biga2
      endif

      sh_a = lam
      sh_i = lam
      sh_H = lam
      
      if (inv .lt. 1.0D-08) then
        getxhp = 1.0D-08
      else       
          getxhp = biga*(sh_a*ab**gam+sh_i*inv**gam+sh_H*hum**gam)**(one/gam)
      endif
      
      return
      end
      
      double precision function getdxhp(inv,hum,ab,nbd)
      use parameters_mod
      use states_mod
      implicit none
      real*8 :: inv,hum,ab,elas,bigf,sh_a,sh_i,sh_H,biga,gam
      integer :: nbd
      
      gam = gam1
      
      if (nbd .eq. 1) then
        if (bigaswitch .eq. 1) then
           biga = biga2
        else
           biga = biga1
        endif
      else
        biga = biga2
      endif

      sh_a = lam
      sh_i = lam
      sh_H = lam
      
      bigf = biga*(sh_a*ab**(gam)+sh_i*inv**gam+sh_H*hum**gam)**((one/gam)-one)
      getdxhp = bigf*one*sh_i*inv**(gam-one)
      
      return
      end
      

	   subroutine func(np,j,x,fj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: fj
      real*8 :: xhp,c,it,phi,utilc,yval,yp,ydp,ab,humstay,getxhp,p
      integer :: khi,klo
 
      it = x(1)
      c = x(2)
      p = pguess(nbdstay)
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))
      
      humstay = hbelief(nbdstay)
      ab = apts(nap)
      xhp = getxhp(it,humstay,ab,nbdstay)
      
      
      if ((nuc .eq. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = dlog(c)+theta*dlog(phi)
         
      elseif ((nuc .ne. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = (c**(one-nuc))/(one-nuc)+theta*dlog(phi)
         
      elseif ((nuc .eq. 1.0D+00) .and. (nus .ne. 1.0D+00)) then
         utilc = dlog(c)+theta*(phi**(one-nus))/(one-nus)
         
      else
         utilc = (c**(one-nuc))/(one-nuc)+theta*(phi**(one-nus))/(one-nus) 
           
      endif
 
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,1,0,0,nap,nbdstay)
       
      fj = -(utilc+beta*yval)
      
      return
      end

	   subroutine func2(np,j,x,fj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: fj
      real*8 :: xhp,c,it,phi,utilc,yval,yp,ydp,ab,humgo,getxhp,p
      integer :: klo,khi
 
      it = x(1)
      c = x(2)
      p = pguess(nbdgo)
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))
      
      humgo = hbelief(nbdgo)
      ab = apts(nap)
      xhp = getxhp(it,humgo,ab,nbdgo)
      
      
      if ((nuc .eq. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = dlog(c)+theta*dlog(phi)
         
      elseif ((nuc .ne. 1.0D+00) .and. (nus .eq. 1.0D+00)) then
         utilc = (c**(one-nuc))/(one-nuc)+theta*dlog(phi)
         
      elseif ((nuc .eq. 1.0D+00) .and. (nus .ne. 1.0D+00)) then
         utilc = dlog(c)+theta*(phi**(one-nus))/(one-nus)
         
      else
         utilc = (c**(one-nuc))/(one-nuc)+theta*(phi**(one-nus))/(one-nus)   
         
      endif

      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,1,0,0,nap,nbdgo)
      
      fj = -(utilc+beta*yval)
     
      return
      end

      subroutine dfunc(np,j,x,fj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: fj
      real*8 :: xhp,c,it,yval,phi,yp,ydp,ab,humstay,getxhp,dhpdi,getdxhp,dphidc,p
      integer :: klo,khi
      external dummy
      
      it = x(1)
      c = x(2)
      p = pguess(nbdstay)
      phi =(theta/p)**(one/nus)*(c**(nuc/nus))
     
      humstay = hbelief(nbdstay)
      ab = apts(nap)
      xhp = getxhp(it,humstay,ab,nbdstay)
      dhpdi = getdxhp(it,humstay,ab,nbdstay)
      dphidc = (nuc/nus)*((theta/p)**(one/nus))*c**((nuc/nus)-one)
      
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,0,1,0,nap,nbdstay)
  
      fj(1) = -beta*yp*dhpdi
      fj(2) = -c**(-nuc)-theta*dphidc*phi**(-nus)
      
      return
      end

      subroutine dfunc2(np,j,x,fj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: fj
      real*8 :: xhp,c,it,yval,yp,ydp,ab,humgo,getxhp,dhpdi,getdxhp,p,phi,dphidc
      integer :: klo,khi
      external dummy
      
      it = x(1)
      c = x(2)
      p = pguess(nbdgo)
      phi =(theta/p)**(one/nus)*(c**(nuc/nus))
      
      humgo = hbelief(nbdgo)
      ab = apts(nap)
      xhp = getxhp(it,humgo,ab,nbdgo)
      
      dhpdi = getdxhp(it,humgo,ab,nbdgo)
      dphidc = (nuc/nus)*((theta/p)**(one/nus))*c**((nuc/nus)-one)
      
      call splint2(hpts,v,vdp,xhp,yval,yp,ydp,khi,klo,0,nhpts,0,1,0,nap,nbdgo)
  
      fj(1) = -beta*yp*dhpdi
      fj(2) = -c**(-nuc)-theta*dphidc*phi**(-nus)
      
      return
      end
    
      subroutine const(np,j,x,gj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: gj
      real*8 :: xhp,c,it,phi,p
      integer :: khi,klo
      
      it = x(1)
      c = x(2)
      p = pguess(nbdstay)
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))

      gj = c+it+p*phi-wage*xhcur
        
      return
      end
      
      
      subroutine const2(np,j,x,gj)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: gj
      real*8 :: xhp,c,it,phi,p
      integer :: khi,klo
      
      it = x(1)
      c = x(2)
      p = pguess(nbdgo)
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))
       
      gj = c+it+p*phi-wage*xhcur+costm
      
      return
      end
                
      subroutine dconst(np,j,x,gj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: gj
      real*8 :: c,p,phi,dphidc
      external dummy
      
      c = x(2)
      p = pguess(nbdstay)
      
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))
      dphidc = (nuc/nus)*((theta/p)**(one/nus))*c**((nuc/nus)-one)
      
      gj(1) = one
      gj(2) = one+p*dphidc

      return
      end
      
      subroutine dconst2(np,j,x,gj,dummy)
      use parameters_mod
      use states_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: gj
      real*8 :: c,p,phi,dphidc
      external dummy
      
      c = x(2)
      p = pguess(nbdgo)
      
      phi = (theta/p)**(one/nus)*(c**(nuc/nus))
      dphidc = (nuc/nus)*((theta/p)**(one/nus))*c**((nuc/nus)-one)
      
      
      gj(1) = one
      gj(2) = one+p*dphidc
      
      return
      end
                                
      double precision function zbrent2(funct,x1,x2,price,tol)
      implicit none
      real*8 :: x1,x2,tol,zip,fa,fb,a,b,c,fc,d,e,tol1,xm,s,p,q,r,price
      integer :: iter
      real*8, parameter :: eps = 3.0D-08
      integer, parameter :: itmax = 300 
      interface
         double precision function funct(x,pr)
         implicit none
         real*8, intent(in) :: x,pr
         end function funct
      end interface

      a = x1
      b = x2
      fa = funct(a,price)
      fb = funct(b,price)
      zip = 0.0D+00

      if ((fa > zip .and. fb > zip) .or. (fa < zip .and. fb < zip)) then
      print*,'root must be bracketed'
      print*, a,b
      print*, fa,fb
      endif

      c = b
      fc = fb
      do iter = 1,itmax
        if (iter .eq. itmax) print*, 'HIT ITMAX'
         if ((fb > zip .and. fc > zip) .or. (fb < zip .and. fc < zip)) then
            c = a
            fc = fa
            d = b-a
            e = d
         endif
         if (dabs(fc) < dabs(fb)) then
            a = b
            b = c
            c = a
            fa = fb
            fb = fc
            fc = fa
         endif
         tol1 = 2.0D+00*eps*dabs(b)+0.5D+00*tol
         xm = 0.5D+00*(c-b)
         if (dabs(xm) <= tol1 .or. fb == zip) then
            zbrent2 = b
            return
         endif
         if (dabs(e) >= tol1 .and. dabs(fa) > dabs(fb)) then
            s = fb/fa
            if (a == c) then
               p = 2.0D+00*xm*s
               q = 1.0D+00-s
            else
               q = fa/fc
               r = fb/fc
               p = s*(2.0D+00*xm*q*(q-r)-(b-a)*(r-1.0D+00))
               q = (q-1.0D+00)*(r-1.0D+00)*(s-1.0D+00)
            endif
            if (p > zip) q = -q
            p = dabs(p)
            if (2.0D+00*p < dmin1(3.0D+00*xm*q-dabs(tol1*q),dabs(e*q))) then
               e = d
               d = p/q
            else
               d = xm
               e = d
            endif
         else
            d = xm
            e = d
         endif
         a = b
         fa = fb
         if (dabs(d) > tol1) then
            b = b+d
         else
            b = b+sign(tol1,xm)
         endif
         fb = funct(b,price)
      end do

      pause 'zbrent2 exceeding maximum iterations'
      zbrent2 = b

      return
      end
      
      double precision function zbrent(funct,x1,x2,tol)
      implicit none
      real*8 :: x1,x2,tol,zip,fa,fb,a,b,c,fc,d,e,tol1,xm,s,p,q,r,price
      integer :: iter
      real*8, parameter :: eps = 3.0D-08
      integer, parameter :: itmax = 300 
      interface
         double precision function funct(x)
         implicit none
         real*8, intent(in) :: x
         end function funct
      end interface

      a = x1
      b = x2
      fa = funct(a)
      fb = funct(b)
      zip = 0.0D+00

      if ((fa > zip .and. fb > zip) .or. (fa < zip .and. fb < zip)) then
      print*,'root must be bracketed'
      print*, a,b
      print*, fa,fb
      endif

      c = b
      fc = fb
      do iter = 1,itmax
        if (iter .eq. itmax) print*, 'HIT ITMAX'
         if ((fb > zip .and. fc > zip) .or. (fb < zip .and. fc < zip)) then
            c = a
            fc = fa
            d = b-a
            e = d
         endif
         if (dabs(fc) < dabs(fb)) then
            a = b
            b = c
            c = a
            fa = fb
            fb = fc
            fc = fa
         endif
         tol1 = 2.0D+00*eps*dabs(b)+0.5D+00*tol
         xm = 0.5D+00*(c-b)
         if (dabs(xm) <= tol1 .or. fb == zip) then
            zbrent = b
            return
         endif
         if (dabs(e) >= tol1 .and. dabs(fa) > dabs(fb)) then
            s = fb/fa
            if (a == c) then
               p = 2.0D+00*xm*s
               q = 1.0D+00-s
            else
               q = fa/fc
               r = fb/fc
               p = s*(2.0D+00*xm*q*(q-r)-(b-a)*(r-1.0D+00))
               q = (q-1.0D+00)*(r-1.0D+00)*(s-1.0D+00)
            endif
            if (p > zip) q = -q
            p = dabs(p)
            if (2.0D+00*p < dmin1(3.0D+00*xm*q-dabs(tol1*q),dabs(e*q))) then
               e = d
               d = p/q
            else
               d = xm
               e = d
            endif
         else
            d = xm
            e = d
         endif
         a = b
         fa = fb
         if (dabs(d) > tol1) then
            b = b+d
         else
            b = b+sign(tol1,xm)
         endif
         fb = funct(b)
      end do

      pause 'zbrent exceeding maximum iterations'
      zbrent = b

      return
      end
      
      double precision function upperb(x,p)
	  use parameters_mod
	  use states_mod
	  implicit none
	  real*8 :: phi
	  real*8, intent(in) :: x,p

      phi = (theta/p)**(one/nus)*(x**(nuc/nus))
	  upperb = p*phi+x-wfix*xhcur

	  return
	  end
	  
	